import sempy
import numpy as np
import scipy.sparse.linalg as spl

import scipy.linalg as sc


class NavierStokes:
    def __init__(self, Space, SpaceGL, Time, u, p, mu = 'none' , 
                  f = 'none' ):
        self.Space = Space
        self.SpaceGL = SpaceGL
        self.u = u
        self.p = p
        self.Time = Time
        
        # Forcing term
        if f == 'none':
            f_dummy = sempy.Function( Space, basis_coeff =  0.0)
            #if self.Space.dim == 1:
            #    b = f_dummy.glob()
            #    self.b = [b]
            if self.Space.dim == 2:
                #b = f_dummy.glob()
                self.f = [f_dummy,f_dummy]
                
            #if self.Space.dim == 3:
            #    b = f_dummy.glob()
            #    self.b = [b,b,b]
        #else:
        #    for i in range(self.Space.dim):
        #        mass_gll = sempy.operators.Mass( self.Space )
        #        self.b[i] = mass_gll.action_local( f[i].basis_coeff )
        # Transport coefficient
        if mu == 'none':
            mu_dummy = sempy.Function( Space, basis_coeff =  1.0 )
            self.mu = mu_dummy
        else:
            self.mu = mu
        # Laplacian
        lap = sempy.operators.Laplacian( self.Space, 
                                         mu = self.mu,
                                         fem_assemble = 'yes' )
        self.A = lap.matrix
        self.A_fem = lap.matrix_fem
        # Convection
        #conv = sempy.operators.Convection( self.Space, 
        #                                  [self.u[0], self.u[1]])
        #self.C = conv.matrix
        # Mass matrix
        mass = sempy.operators.Mass( self.Space )
        self.M = mass.matrix
        self.M_inv = mass.matrix_inv
        # Helmholtz
        self.H = sempy.operators.MultipleOperators( [self.M,self.A],
                                                    [1.0,self.Time.h],
                                                    assemble = 'no').matrix
        #self.H_fem = sempy.operators.MultipleOperators( [self.M,self.A_fem],
        #                                            [1.0,self.Time.h],
        #                                            assemble = 'yes').matrix
        # Precond.
        self.P = sempy.precond.Preconditioner( self.Space, [self.M, self.A_fem], 
                                          scaling_factor=[1.0,self.Time.h],
                                          drop_tol = 0.5 ).matrix
        # Divergence
        self.__divergence_matrix()
        # Gradient
        self.__gradient_matrix()
        # Uzawa operator
        self.U = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__action_uzawa,
                            dtype = 'float' )
        self.U_fem = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__action_uzawa_fem,
                            dtype = 'float' )
        self.U_pre = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__uzawa_pre,
                            dtype = 'float' )
        #print 'U_pre'
        #print self.U_pre*self.p.glob()
        # Linear solvers
        self.inner_solver = sempy.linsolvers.Krylov(tol = 1e-10,
                                                    solver_type = 'bicgstab',
                                                    pre = self.P )
        self.outer_solver = sempy.linsolvers.Krylov(
                                            tol = 1e-08,
                                            solver_type = 'cg',
                                            pre = self.U_pre)
        self.pre_solver  = sempy.linsolvers.Krylov(tol = 1e-10,
                                         solver_type = 'bicgstab')
                                         #pre = self.P )
        
    def __mom_b(self,u,f):
        
        conv = sempy.operators.Convection( self.Space, [ u[0], u[1] ] )
        C = conv.matrix
            
        b_x = self.M*f[0].glob() - C*u[0].glob()
        b_y = self.M*f[1].glob() - C*u[1].glob()
        b = [b_x, b_y]
        
        return b    
    
    def __action_uzawa(self,q_in):
        #print 'action uzawa...'
        #
        if self.Space.dim == 2:
            # 1
            s_x = self.G_x*q_in
            s_y = self.G_y*q_in
            # 2
            t_x = sempy.Function( self.Space, basis_coeff = 0.0 )
            t_y = sempy.Function( self.Space, basis_coeff = 0.0 )
            [v_x, flag] = self.inner_solver.solve( self.H, s_x, t_x.glob() )
            #[v_x, flag] = spl.cg( self.H, s_x, x0 = t_x.glob(), 
            #                      tol = 1e-15,
            #                      maxiter = 1000,
            #                      M= self.P)
            print'v_x flag=',flag 
            #print '3 uz cg=',flag
            [v_y, flag] = self.inner_solver.solve( self.H, s_y, t_y.glob() )
            #[v_y, flag] = spl.cg( self.H, s_y, x0 = t_y.glob(), 
            #                      tol = 1e-15,
            #                      maxiter = 1000,
            #                      M= self.P)
            print'v_y flag=',flag 
            # 3
            q_out = self.D_x * v_x + self.D_y * v_y
            
        return q_out
    
    def __uzawa_pre(self,q_in):
        print 'pre uzawa...'
        s = sempy.Function( self.SpaceGL, basis_coeff = 0.0 )
        #[p_out, flag] = spl.bicgstab( self.U_fem, q_in, #x0 = s.glob(), 
        #                        tol = 1e-15,
        #                        maxiter = 2000)
        #                       #callback = self.cb.it_count )
        [p_out, flag] = self.pre_solver.solve( self.U_fem, q_in, s.glob() )
        #[p_out, flag] = sempy.linsolvers.Krylov(tol = 1e-15).
        #solve( self.U_fem, q_in, s.glob() )
        print 'pre uzawa flag=',flag
        #t = self.U_fem*p_out - q_in
        #print 'pre uzawa norm', sc.norm(t,1)
        print 'pre uzawa stop'
        return p_out
        
        
    def __action_uzawa_fem(self,q_in):
        #
        if self.Space.dim == 2:
            # 1
            s_x = self.G_x * q_in
            s_y = self.G_y * q_in
            # 2
            v_x = self.P * s_x
            v_y = self.P * s_y
            # 3
            q_out = self.D_x * v_x + self.D_y * v_y
            
        return q_out
         
    def __divergence_matrix(self):
        # Divergence
        div = sempy.fluid.Divergence( self.Space, self.SpaceGL )
        
        if self.Space.dim == 2:
            
            self.D_x = div.matrix_x
            self.D_y = div.matrix_y
            
        if self.Space.dim == 3:
            
            self.D_x = div.matrix_x
            self.D_y = div.matrix_y
            self.D_z = div.matrix_z
            
    def __gradient_matrix(self):
        grad = sempy.fluid.Gradient( self.Space, self.SpaceGL )
        
        if self.Space.dim == 2:
            
            self.G_x = grad.matrix_x
            self.G_y = grad.matrix_y
            
        if self.Space.dim == 3:
            
            self.G_x = grad.matrix_x
            self.G_y = grad.matrix_y
            self.G_z = grad.matrix_z
        
    def solve(self):
        '''
        Solve.
        '''
        
        #-- Output
        self.u.to_file()
        self.p.to_file()
        #ystart = np.zeros( (self.Space.dof), float )
        #print self.H*ystart
        #
        #ystart[:] = self.y0.glob()
        ustart = self.u
        pstart = self.p
        pend = sempy.Function( self.SpaceGL, basis_coeff = 0.0 )
        uend = sempy.VectorFunction( self.Space )
        #-- Time-stepping
        for i in range(self.Time.time_steps):
            print '--> time step no=',i+1
            uend, pend = self.__time_stepping( ustart, uend, pstart, pend )
            self.Time.increment()
            ustart = uend
            pstart = pend
            #self.u = ustart
            #self.p = pstart
            self.u[0].basis_coeff =ustart[0].basis_coeff
            self.u[1].basis_coeff =ustart[1].basis_coeff
            self.p.basis_coeff = pstart.basis_coeff
            self.u.to_file()
            self.p.to_file()
            print 'time=',self.Time.time
        return self.u, self.p
            
    def __time_stepping(self,ustart,uend, pstart,pend):
        # The b term in Navier-Stokes
        b = self.__mom_b( ustart, self.f )
        # RHS Uzawa
        b_uz = self.__rhs_uzawa(ustart, b)
        # Solve Uzawa
        print 'solving uzawa'
        [p_glob, flag] = self.outer_solver.solve( self.U, b_uz, 
                                                  pstart.glob() )
        #print 'U_pre*b=',self.U_pre*b_uz
        
        #[p_glob, flag] = spl.cg( self.U, b_uz, 
        #                         x0 = pstart.glob(), 
        #                         tol = 1e-10,
        #                         maxiter = 2000,
        #                         M = self.U_pre)
        pend.basis_coeff = self.SpaceGL.mapping_q( p_glob )
        print 'solving uzawa stop. flag=',flag
        # Momentum
        if self.Space.dim == 2:
            # u  
            b_x = self.M*ustart[0].glob() + \
                  self.Time.h*( b[0] - self.G_x*p_glob )
            #[u_x, flag] = self.inner_solver.solve( self.H, b_x, 
            #                                       ustart[0].glob() )
            [u_x, flag] = spl.cg( self.H, b_x, x0 = ustart[0].glob(), 
                                  tol = 1e-15,
                                  maxiter = 1000,
                                  M = self.P)
            print 'x mom flag=',flag
            uend[0].basis_coeff = self.Space.mapping_q(u_x)
            # v
            b_y = self.M * ustart[1].glob() + \
                  self.Time.h*( b[1] - self.G_y*p_glob )
                  
            #[u_y, flag] = self.inner_solver.solve( self.H, b_y, 
            #                                   ustart[1].glob() )
            [u_y, flag] = spl.cg( self.H, b_y, x0 = ustart[1].glob(), 
                                  tol = 1e-15,
                                  maxiter = 1000,
                                  M = self.P)
            print 'y mom flag=',flag
            uend[1].basis_coeff = self.Space.mapping_q(u_y)
        
        return uend, pend
                    
    def __rhs_uzawa(self,u, b):
        print 'rhs uzawa..'
        if self.Space.dim == 2:
            
            # u component
            b_x = self.M * u[0].glob() + self.Time.h * b[0]
            [t_x, flag] = self.inner_solver.solve( self.H, b_x, u[0].glob() )
            #[t_x, flag] = spl.cg( self.H, b_x, x0 = u[0].glob(), 
            #                      tol = 1e-15,
            #                      maxiter = 1000,
            #                      M = self.P)
            print 't_x rhs flag=',flag
            # v component
            b_y = self.M * u[1].glob() + self.Time.h * b[1]
            [t_y, flag] = self.inner_solver.solve( self.H, b_y, u[1].glob() )
            #[t_y, flag] = spl.cg( self.H, b_y, x0 = u[1].glob(), 
            #                      tol = 1e-15,
            #                      maxiter = 1000,
            #                      M = self.P)
            print 't_y rhs flag=',flag
            # Divergence
            b_uz = ( 1.0 / self.Time.h ) * ( self.D_x*t_x + self.D_y*t_y )
            #print '2 rhs cg=',flag
            
        #if self.Space.dim == 3:
        #    # u component
        #    [t_x, flag] = self.inner_solver.solve( self.A, 
        #                            self.b[0], self.u[0].glob())
        #    # v component
        #    [t_y, flag] = self.inner_solver.solve( self.A, 
        #                            self.b[1], self.u[1].glob())
        #    # w component
        #    [t_z, flag] = self.inner_solver.solve( self.A, 
        #                            self.b[2], self.u[2].glob())
        #    # Divergence
        #    b = self.D_x*t_x + self.D_y*t_y + self.D_z*t_z
        #print '..rhs.'
        print 'rhs uzawa stop.'    
        return b_uz
        
        


'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 

import sempy
import numpy as np


class VectorFunction:
    '''
    A vector function used to represent e.g. the velocity vector field. 
    
    .. math::
    
       \\mathbf u\in (H^1(\Omega))^d
    
    :param Space: The function space.
    :type Space: :class:`sempy.Space`
    :param filename: The filename. 
    :type filename: string
    '''
    def __init__( self, Space, filename = None ):
        self.Space = Space
        self.filename = filename
        self.file_count = 0
        
        self.U= []
        if self.Space.dim == 1:
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                      filename = 'none' ))
            self.U.append('none')
            self.U.append('none')
            
        if self.Space.dim == 2:
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                          filename = 'none' ))
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                          filename = 'none' ))
            self.U.append('none')
            
        if self.Space.dim == 3:
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                          filename = 'none' ))
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                          filename = 'none' ))
            self.U.append(sempy.Function( self.Space, basis_coeff = 0.0, 
                                          filename = 'none' ))
    #def __setitem__(self,index,value):
    #    self.U[index].basis_coeff = value
    #    return self.U[index]
    
    def __getitem__(self,index):
       return self.U[index]
    
    def vorticity(self):
        #
        if self.Space.dim ==2:
            [u_x,u_y] = self.U[0].grad()
            [v_x,v_y] = self.U[1].grad()
            #omega = sempy.Function( self.Space , basis_coeff = v_x-u_y )
            #omega.plot( legend = legend, filled = filled,
            #            resolution = resolution, 
            #            show_elements = show_elements,
            #            font_size = font_size, show = show,
            #            lower=lower, upper = upper )
            return v_x-u_y
        
    def plot_vorticity(self,legend=True,filled=True,resolution=20,
                       show_elements=True,font_size=20, show=True,
                       lower='none',upper='none'):
        '''
        Plots the vorticity :math:`\omega`:  
        
        .. math::
        
           \\omega= \\frac{\\partial v}{\\partial x}-
                    \\frac{\\partial u}{\\partial y}
        '''
        if self.Space.dim ==2:
            [u_x,u_y]=self.U[0].grad()
            [v_x,v_y]=self.U[1].grad()
            omega = sempy.Function( self.Space , basis_coeff = v_x-u_y )
            omega.plot( legend = legend, filled = filled,
                        resolution = resolution, 
                        show_elements = show_elements,
                        font_size = font_size, show = show,
                        lower=lower, upper = upper )
        else:
            print 'Vorticity plot not implemented for 1D and 2D.'
        
        
    def plot_stream_function(self, legend = True, filled = True,
                             resolution = 20, show_elements = True,
                             font_size = 20, show = False,
                             show_boundary=False,
                             lower='none',upper='none'):
        '''
        Plots the stream-function :math:`\psi`.
        
        .. math::
        
           -\\nabla^2 \psi=\\omega
        
        where the vorticity :math:`\omega` is  
        
        .. math::
        
           \\omega= \\frac{\\partial v}{\\partial x}-
                    \\frac{\\partial u}{\\partial y}
        
        and the boundary conditions are
        
        .. math::
           \\frac{\\partial\psi}{\\partial x}\\cdot\\mathbf n\\bigg|_\Gamma=
           -v
           
           \\frac{\\partial\psi}{\\partial y}\\cdot\\mathbf n\\bigg|_\Gamma=
           u
        '''
        if self.Space.dim ==1:
            print '***: Calculation of the stream-function is not'
            print '    implemented for 1D.'
        if self.Space.dim ==2:
            
            # Vorticity, omega
            [u_x,u_y]=self.U[0].grad()
            [v_x,v_y]=self.U[1].grad()
            omega = sempy.Function( self.Space , 
                                    basis_coeff = v_x-u_y )
            # Weak BC
            weak_x = sempy.BoundaryFunction( self.Space, mask = 'no' ) 
            weak_y = sempy.BoundaryFunction( self.Space, mask = 'no' ) 
            for i in range(5):
                weak_x.set_value(-self.U[1].basis_coeff,i)
                weak_y.set_value(self.U[0].basis_coeff,i)
            #print 'weak_x=',weak_x.basis_coeff*self.Space.n_unit[1]
            weak_x.basis_coeff = weak_x.basis_coeff*self.Space.n_unit[0]
            weak_y.basis_coeff = weak_y.basis_coeff*self.Space.n_unit[1]
            g_neu = sempy.BoundaryFunction( self.Space, mask = 'no' )
            g_neu.basis_coeff = weak_x.basis_coeff + weak_y.basis_coeff
            
            # Lapalcian
            lap = sempy.operators.Laplacian( self.Space, g_neu = g_neu,
                                             mask = 'no',
                                             fem_assemble = 'yes' )
            A = lap.matrix
            b_weak = lap.linear_form
            
            # RHS
            mass = sempy.operators.Mass( self.Space, mask = 'no' )
            M = mass.matrix
            b = M*omega.glob() + b_weak
            print 'sum b1=',b.sum()
            alpha=0.0
            for k in range(self.Space.dof):
                alpha=alpha+b[k]
            b=b-alpha/float(self.Space.dof)
            print 'sum b2=',b.sum()
            # Solver
            solver = sempy.linsolvers.Krylov(tol = 1e-09,
                                    solver_type = 'cg',
                                    maxiter = 2000)
                                    #pre = P_A )
            # Solve
            s = np.zeros( (self.Space.dof), float )
            psi = sempy.Function( self.Space )
            s, iter = solver.solve(A,b,s)
            psi = sempy.Function( self.Space, 
                                  basis_coeff = self.Space.mapping_q( s )  )
            print 'converged at iter=',iter
            #psi.plot_wire()
            #print psi.basis_coeff
            psi.plot( legend = legend, filled = filled, 
                      resolution = resolution, show_elements = show_elements,
                      font_size = font_size, show = show,
                      show_boundary=show_boundary,
                      lower = lower, upper = upper)
            
        if self.Space.dim ==3:
            print '***: Calculation of the stream-function is not'
            print '    implemented for 3D.'
            
    def calc_stream_function(self):
        '''
        Calculates the stream function.
        '''
        if self.Space.dim ==1:
            print '***: Calculation of the stream-function is not'
            print '    implemented for 1D.'
        if self.Space.dim ==2:
            
            # Vorticity, omega
            [u_x,u_y]=self.U[0].grad()
            [v_x,v_y]=self.U[1].grad()
            omega = sempy.Function( self.Space , 
                                    basis_coeff = v_x-u_y )
            # Weak BC
            weak_x = sempy.BoundaryFunction( self.Space, mask = 'no' ) 
            weak_y = sempy.BoundaryFunction( self.Space, mask = 'no' ) 
            for i in range(5):
                weak_x.set_value(-self.U[1].basis_coeff,i)
                weak_y.set_value(self.U[0].basis_coeff,i)
            #print 'weak_x=',weak_x.basis_coeff*self.Space.n_unit[1]
            weak_x.basis_coeff = weak_x.basis_coeff*self.Space.n_unit[0]
            weak_y.basis_coeff = weak_y.basis_coeff*self.Space.n_unit[1]
            g_neu = sempy.BoundaryFunction( self.Space, mask = 'no' )
            g_neu.basis_coeff = weak_x.basis_coeff + weak_y.basis_coeff
            
            # Lapalcian
            lap = sempy.operators.Laplacian( self.Space, g_neu = g_neu,
                                             mask = 'no',
                                             fem_assemble = 'yes',
                                             silent = True )
            A = lap.matrix
            b_weak = lap.linear_form
            
            # RHS
            mass = sempy.operators.Mass( self.Space, mask = 'no' )
            M = mass.matrix
            b = M*omega.glob() + b_weak
            print 'sum b1=',b.sum()
            alpha=0.0
            for k in range(self.Space.dof):
                alpha=alpha+b[k]
            b=b-alpha/float(self.Space.dof)
            print 'sum b2=',b.sum()
            # Solver
            solver = sempy.linsolvers.Krylov(tol = 1e-09,
                                    solver_type = 'cg',
                                    maxiter = 2000)
                                    #pre = P_A )
            # Solve
            s = np.zeros( (self.Space.dof), float )
            psi = sempy.Function( self.Space )
            s, iter = solver.solve(A,b,s)
            psi = sempy.Function( self.Space, 
                                  basis_coeff = self.Space.mapping_q( s )  )
            print 'Calc. stream func. conv. at it.=',iter
            return psi.basis_coeff
            
        if self.Space.dim ==3:
            print '***: Calculation of the stream-function is not'
            print '    implemented for 3D.'
    
                    
    def to_file(self):
        '''
        Print to file.
        '''
        if not self.filename == None:
            if self.Space.dim == 2:
                print 'to file=',self.filename
                self.__to_file_2d()
            
            if self.Space.dim == 3:
                self.__to_file_3d()
    
    def __to_file_2d(self):
        '''
        Print the basis coefficients to a file in the vtk file format.
        '''
        # Filename
        #if self.file_count == 0:
        #    _filename = self.filename + '.vtk'
        #    self.file_count=self.file_count+1
        #else:
        
        _filename = self.filename + str(self.file_count) + '.vtk'
        self.file_count=self.file_count+1
        
        #print '_filename=',_filename
        
        # open file
        f = open(_filename,'w')
        f.write('# vtk DataFile Version 3.0\n')
        f.write('Sempy data file\n')
        f.write('ASCII\n')
        f.write('DATASET UNSTRUCTURED_GRID\n')
        
        #--- Points
        f.write('\n')
        f.write('POINTS ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.dof
        f.write(np.array_str(a[0]))
        f.write(' float\n')
        
        
        xx = self.Space.fem.local_to_global(self.Space.fem.x)
        yy = self.Space.fem.local_to_global(self.Space.fem.y)
        zz = np.zeros((self.Space.dof),float)

        for k in range(self.Space.fem.dof):
            f.write(np.array_str(xx[k]))
            f.write(' ')
            f.write(np.array_str(yy[k]))
            f.write(' ')
            f.write(np.array_str(zz[k]))
            f.write(' \n')

        #--- Cells
        f.write('\n')
        f.write('CELLS ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.noe
        f.write( np.array_str( a[0] ) )
        f.write( ' ' ) 
        a = np.zeros( 1, np.int )
        a[0] = self.Space.fem.noe*(self.Space.fem.n*self.Space.fem.n + 1)
        f.write( np.array_str( a[0] ) )
        f.write('\n')
        for k in range(self.Space.fem.noe):
            f.write('4 ')
            f.write(np.array_str(self.Space.fem.theta[k,0,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,0,1]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,1]))
            f.write(' \n')
            
        #--- Cell types
        f.write( '\n' )
        f.write( 'CELL_TYPES ' )
        a = np.zeros( 1, np.int )
        a[0] = self.Space.fem.noe
        f.write( np.array_str( a[0] ) )
        f.write( '\n' )
        a[0] = 8
        for k in range(self.Space.fem.noe):
            f.write(np.array_str(a[0]))
            f.write(' \n')
        f.write('\n')
        
        #--- Scalar
        #if self.file_count > 0:
        __u = self.Space.fem.sem_to_fem(self.U[0].basis_coeff)
        __v = self.Space.fem.sem_to_fem(self.U[1].basis_coeff)
        _u = self.Space.fem.local_to_global(__u)
        _v = self.Space.fem.local_to_global(__v)
        _w = np.zeros((self.Space.dof),float)
    
        #_u = self.Space.fem.local_to_global(__u)#self.basis_coeff)
        
        f.write('POINT_DATA ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.dof
        f.write( np.array_str(a[0]) )
        f.write('\n')
        f.write('VECTORS stians float\n')
        #a=np.zeros(1,np.float)
        #a[0]=3.0
        for k in range(self.Space.fem.dof):
            f.write(np.array_str(_u[k]))
            f.write(' ')
            f.write(np.array_str(_v[k]))
            f.write(' ')
            f.write(np.array_str(_w[k]))
            f.write(' \n')
        f.write('\n')
    
        f.close()
    
            
    def __to_file_3d(self):
        '''
        Print the basis coefficients to a file in the vtk file format.
        '''
        # Filename
        #if self.file_count == 0:
        #    _filename = self.filename + '.vtk'
        #    self.file_count=self.file_count+1
        #else:
        
        _filename = self.filename + str(self.file_count) + '.vtk'
        self.file_count=self.file_count+1
        #print '_filename=',_filename
        
        # open file
        f = open(_filename,'w')
        f.write('# vtk DataFile Version 3.0\n')
        f.write('Sempy data file\n')
        f.write('ASCII\n')
        f.write('DATASET UNSTRUCTURED_GRID\n')
        
        #--- Points
        f.write('\n')
        f.write('POINTS ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.dof
        f.write(np.array_str(a[0]))
        f.write(' float\n')
        
        
        xx = self.Space.fem.local_to_global(self.Space.fem.x)
        yy = self.Space.fem.local_to_global(self.Space.fem.y)
        zz = self.Space.fem.local_to_global(self.Space.fem.z)
        
        for k in range(self.Space.fem.dof):
            f.write(np.array_str(xx[k]))
            f.write(' ')
            f.write(np.array_str(yy[k]))
            f.write(' ')
            f.write(np.array_str(zz[k]))
            f.write(' \n')
        
        #--- Cells
        f.write('\n')
        f.write('CELLS ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.noe
        f.write( np.array_str( a[0] ) )
        f.write( ' ' ) 
        a = np.zeros( 1, np.int )
        a[0] = self.Space.fem.noe*(self.Space.fem.n*self.Space.fem.n*self.Space.fem.n + 1)
        f.write( np.array_str( a[0] ) )
        f.write('\n')
        for k in range(self.Space.fem.noe):
            f.write('8 ')
            f.write(np.array_str(self.Space.fem.theta[k,0,0,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,0,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,0,1]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,0,0,1]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,0,1,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,1,0]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,1,1,1]))
            f.write(' ')
            f.write(np.array_str(self.Space.fem.theta[k,0,1,1]))
            f.write(' \n')
            
        #--- Cell types
        f.write( '\n' )
        f.write( 'CELL_TYPES ' )
        a = np.zeros( 1, np.int )
        a[0] = self.Space.fem.noe
        f.write( np.array_str( a[0] ) )
        f.write( '\n' )
        a[0] = 12
        for k in range(self.Space.fem.noe):
            f.write(np.array_str(a[0]))
            f.write(' \n')
        f.write('\n')
        
        #--- Scalar
        #if self.file_count > 0:
        __u = self.Space.fem.sem_to_fem(self.U[0].basis_coeff)
        __v = self.Space.fem.sem_to_fem(self.U[1].basis_coeff)
        __w = self.Space.fem.sem_to_fem(self.U[2].basis_coeff)
        
        _u = self.Space.fem.local_to_global(__u)
        _v = self.Space.fem.local_to_global(__v)
        _w = self.Space.fem.local_to_global(__w)
        
     
        #f.write('LOOKUP_TABLE default\n')    
        f.write('POINT_DATA ')
        a = np.zeros(1,np.int)
        a[0] = self.Space.fem.dof
        f.write( np.array_str(a[0]) )
        f.write('\n')
        f.write('VECTORS stians float\n')
    
        #a=np.zeros(1,np.float)
        #a[0]=3.0
        for k in range(self.Space.fem.dof):
            f.write(np.array_str(_u[k]))
            f.write(' ')
            f.write(np.array_str(_v[k]))
            f.write(' ')
            f.write(np.array_str(_w[k]))
            #f.write(np.array_str(a[0]))
            f.write(' \n')
        f.write('\n')
    
        f.close()
       
    
            
        
        


import sempy
from prk import PRK
import numpy as np


def __bc__(y,t):
    return y

class Strang(PRK):
    '''
    Implementation of an operator splitting approach known as Strang splitting. 
    The method solves a system of ODEs 
    
    .. math::
       M\\frac{dy}{dt}=Ly+N(y,t)
    
    where :math:`L` is a linear operator and :math:`N(y,t)` is a non-linear 
    function. 
    
    The first order method consists of solving two sub-problems:
    
    .. math::
    
       \\frac{d\\tilde y}{dt}=N(y,t),\quad {\\tilde y}_0=y^{n},\quad 
       t^{n} < t \\le t^{n+1}
       
       \\frac{d\\tilde{\\tilde y}}{dt}=Ly,\quad
       {\\tilde{\\tilde{y}}}_0={\\tilde y}(t^{n+1}),\quad 
       t^{n} < t \\le t^{n+1}
       
       y^{n+1}={\\tilde{\\tilde{y}}}(t^{n+1})
    
    The second order method is similar, but here the symmetry increases 
    the accuracy:
    
    .. math::
    
       \\frac{d\\tilde y}{dt}=N(\\tilde y,t),\quad 
       {\\tilde y}_0=y^{n},\quad 
       t^{n} < t \\le t^{n+1/2}
       
       \\frac{d\\tilde{\\tilde y}}{dt}=Ly,\quad
       {\\tilde{\\tilde{y}}}_0={\\tilde y}(t^{n+1}),\quad 
       t^{n} < t \\le t^{n+1}
       
       \\frac{d\\tilde y}{dt}=N(\\tilde y,t),\quad 
       {\\tilde y}_0={\\tilde{\\tilde{y}}}(t^{n+1}),\quad 
       t^{n+1/2} < t \\le t^{n+1}
       
       y^{n+1}= \\tilde{y}(t^{n+1})
       
    :param y: Initial condition
    :type y: A collection of :class:`sempy.Function`
    :param Time: An instance of the :class:`sempy.ode.Time` class.
    :kwargs: * **L** - Linear operator
             * **L_pre** - The FEM equivalent of L. Used to create 
               preconditioner. 
             * **force_function** - A function, possibly non-linear. 
               This is the function :math:`N(y,t)` in the equation above.
             * **bound_cond** - Strong BCs.
             * **order** (*int*) - Order of the method. Either 1 or 2. 
             * **explicit_steps** (*int*) -  Number of steps used in the 
               explicit solver. 
             * **linear_solver** - The linear solver. The default is 
               CG. If L is not SPD, then one can for example use bicgstab.
             * **iter_tol** (*float*) - Iterative tolerance.
    '''
    def __init__(self, y, Time, L = None, L_pre= None,
                 force_function = None, bound_cond = __bc__,
                 order = 1, explicit_steps = 1,
                 file_increment = 1000,hyper_filter = False,
                 call_back=None,
                 linear_solver = None, iter_tol = 1e-10, maxiter = 1000):
        PRK.__init__( self, y, Time, L = L, L_pre= L_pre,
                      force_function = force_function, 
                      bound_cond = bound_cond, 
                      file_increment=file_increment,
                      hyper_filter = hyper_filter,call_back=call_back,
                      linear_solver = linear_solver, iter_tol = iter_tol,
                      maxiter = maxiter)
        self.explicit_steps = explicit_steps
        self.order = order
        
    def solve(self):
        '''
        Solves the ODE system.
        '''
        if self.order == 1:
            y = self.solve_1st()
            return y
        
        if self.order == 2:
            y = self.solve_2nd()
            return y
                    
    def solve_1st(self):
        #-- Solution vector y: y[0]=y^{n+1}, 
        ystart = self.y0
        #
        irk = sempy.ode.PRK( ystart, self.Time, 
                             L = self.L_in, L_pre = self.L_pre,
                             bound_cond = self.bound_cond,
                             prk_method = 'dirk111',
                             iter_tol = self.iter_tol,
                             maxiter=self.maxiter )
        prk = sempy.ode.PRK( ystart,self.Time,
                             force_function = self.force_function, 
                             bound_cond = self.bound_cond,
                             prk_method='erk4')
        # -- Callback
        if not self.call_back == None: 
            self.call_back(self.y0, self.Time.time,0)
        #-- Time-stepping
        for i in range(self.Time.time_steps):
            local_time = self.Time.time
            h = self.Time.h
            # Solve non-linear problem explicitly
            Time_local = sempy.ode.Time(self.Time.Space,
                           start_time = local_time,
                           end_time = local_time + h,
                           time_steps = self.explicit_steps)
            prk.y0 = ystart
            prk.Time = Time_local
            yend = prk.solve()
            # Solve linear problem implicitly
            #y_temp1 = self.component_to_vector( yend )
            #y_temp  = self.apply_bc(y_temp1,local_time)
            #self.vector_to_component(y_temp)
            
            Time_local = sempy.ode.Time(self.Time.Space,
                          start_time = local_time,
                          end_time = local_time + h,
                          time_steps = 1)
            
            irk.y0 = self.bound_cond(yend,local_time)#self.y_out
            irk.Time = Time_local
            ystart = irk.solve()
            self.Time.increment()
            print '--> Time step number: ', i+1,'time=',self.Time.time
            # -- Callback
            if not self.call_back == None:
                self.call_back(ystart,self.Time.time,i+1)
            
        return ystart
    
    def solve_2nd(self):
        #-- Solution vector y: y[0]=y^{n+1}, 
        ystart = self.y0
        # Use only one time-step for the implicit solver. Otherwise
        # implement complex BCs.
        irk = sempy.ode.IMEX( ystart, self.Time, 
                              L = self.L_in, L_pre = self.L_pre,
                              bound_cond = self.bound_cond,
                              imex_method = 'cn',
                              iter_tol = self.iter_tol,
                              maxiter=self.maxiter  )
        Time_local = sempy.ode.Time(self.Time.Space,
                           start_time = 0.0,
                           end_time = 0.5*self.Time.h,
                           time_steps = self.explicit_steps)
        erk = sempy.ode.PRK( ystart,Time_local,
                             force_function = self.force_function, 
                             bound_cond = self.bound_cond,
                             prk_method='erk4')
        # -- Callback
        if not self.call_back == None: 
            self.call_back(self.y0, self.Time.time,0)
        #-- Time-stepping
        for i in range(self.Time.time_steps):
            local_time = self.Time.time
            h = self.Time.h
            # 1) Solve non-linear problem explicitly
            Time_local = sempy.ode.Time(self.Time.Space,
                           start_time = local_time,
                           end_time = local_time + 0.5*h,
                           time_steps = self.explicit_steps)
            erk.y0 = ystart
            erk.Time = Time_local
            yend = erk.solve()
            # 2) Solve linear problem implicitly
            #--BC
            Time_local = sempy.ode.Time(self.Time.Space,
                          start_time = local_time,
                          end_time = local_time + h,
                          time_steps = 1)
            irk.y0 = self.bound_cond(yend,local_time)
            irk.Time = Time_local
            ystart = irk.solve()
            # 3) Solve non-linear problem
            Time_local = sempy.ode.Time(self.Time.Space,
                           start_time = local_time+0.5*h,
                           end_time = local_time + h,
                           time_steps = self.explicit_steps)
            erk.y0 = self.bound_cond(ystart,local_time+0.5*h)
            erk.Time = Time_local
            yend = erk.solve()
            # Time increment
            self.Time.increment()
            ystart = yend
            print '--> Time step number: ', i+1,'time=',self.Time.time
            # -- Callback
            if not self.call_back == None:
                self.call_back(ystart,self.Time.time,i+1)
            
        return ystart
    
    


'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 

import sempy
import scipy.sparse.linalg as spl
import numpy as np


class SteadyStokes:
    '''
    A solver for the steady Stokes problem
    
    .. math::
    
       -\\nabla\cdot\mu\\nabla \mathbf u +\\nabla p =\mathbf f
       
       \\nabla\cdot\mathbf u = 0
    
    which reads
    
    .. math:: 
       :nowrap:
        
        \\begin{equation}
        \\begin{bmatrix}
        \underline{A} & \underline{G}\\\\
        \underline{D} & 0
        \\end{bmatrix}
        \\begin{bmatrix}
        \underline{u}\\\\
        \underline{p}
        \\end{bmatrix}
        =
        \\begin{bmatrix}
        \underline{b}\\\\
        0
        \\end{bmatrix}
        \\nonumber
        \\end{equation}
    
    in discrete form. One way to solve this system is by using 
    the Uzawa method, i.e.:    
    
    1. Solve the Uzawa equation for pressure :math:`p` 
    
       .. math::
          
          \underline{D}\,\underline{A}^{-1}\underline{G}
          \,\underline{p} = 
          \underline{D}\,\underline{A}^{-1}\underline{b}
    
    2. Solve the momentum equation for the velocity field :math:`u` 
    
       .. math::
       
          \underline{A}\,\underline{u}+
          \underline{G}\,\underline{p}=\underline{b}
    
    Here, the first step involves nested iterations. 
    
    :param Space: Discrete space.
    :type Space: :class:`sempy.Space`
    :param SpaceGL: Discrete space.
    :type SpaceGL: :class:`sempy.fluid.SpaceGL`
    :param u: Velocity. A function in :math:`(H^1(\Omega))^d`.
    :type u: :class:`sempy.VectorFunction`
    :param p: Pressure. A function in :math:`L^2(\Omega)`.
    :type p: :class:`sempy.Function`
    :param mu: Viscosity. If :literal:`None` it will be set to unity. 
    :type mu: :class:`sempy.Function`
    :param f: Forcing term. If :literal:`None` it will be set to zero. 
    :type f: :class:`sempy.Function`
    
    :attribute: * **U** - The uzawa operator 
                  :math:`\underline{U}=\underline{D}\,\underline{A}^{-1}\underline{G}`
    
    **Example**::
    
       import sempy
       
       X = sempy.Space( filename = 'cube', n = 4, dim = 3 )
       X_gl = sempy.fluid.SpaceGL( X )

       U = sempy.VectorFunction( X, filename = 'vecfunc' )
       U[0].set_bc( 1.0, 1 )
       p = sempy.Function( X_gl, filename = 'pressure' )

       solver = sempy.solvers.SteadyStokes( X, X_gl, U, p )
       p, U, flag = solver.solve()
       
       print 'Number of outer iter=',flag
       U.to_file()
       
    '''
    def __init__( self, Space, SpaceGL, u, p, mu = None, 
                  f = None ):
        self.Space = Space
        self.SpaceGL = SpaceGL
        self.u = u
        self.p = p
        
        # Forcing term
        if f == None:
            f_dummy = sempy.Function( Space, basis_coeff =  0.0)
            if self.Space.dim == 1:
                b = f_dummy.glob()
                self.b = [b]
            if self.Space.dim == 2:
                b = f_dummy.glob()
                self.b = [b,b]
            if self.Space.dim == 3:
                b = f_dummy.glob()
                self.b = [b,b,b]
        else:
            mass_gll = sempy.operators.Mass( self.Space )
            #f_dummy = sempy.Function( Space, basis_coeff =  0.0)
            if self.Space.dim == 2:
                s1 = np.zeros( ( self.Space.dof ) )
                s2 = np.zeros( ( self.Space.dof ) )
                #b = f_dummy.glob()
                self.b = [ s1, s2 ]
                for i in range(self.Space.dim):
                    self.b[i] = mass_gll.action_local( f[i].basis_coeff )
                
        # Transport coefficient
        if mu == None:
            mu_dummy = sempy.Function( Space, basis_coeff =  1.0 )
            self.mu = mu_dummy
        else:
            self.mu = mu
        
        # Laplacian
        lap = sempy.operators.Laplacian( self.Space, 
                                         mu = self.mu,
                                         fem_assemble = 'yes' )
        self.A = lap.matrix
        # Preconditioner
        A_fem = lap.matrix_fem
        self.P = sempy.precond.Preconditioner( self.Space, [A_fem], 
                                          drop_tol = 0.5 ).matrix
        # Gradient
        grad = sempy.fluid.Gradient( self.Space, self.SpaceGL )
        self.G = grad.matrix

        # Divergence
        div = sempy.fluid.Divergence( self.Space, self.SpaceGL )
        self.D = div.matrix
        
        # Solution vector
        self.inner_solver = sempy.linsolvers.Krylov( tol = 1e-15,
                                                     solver_type = 'bicgstab',
                                                     pre = self.P )
        self.M_gl_inv = sempy.precond.Preconditioner( 
                                    self.SpaceGL, 
                                    [self.SpaceGL.mass_matrix] ).matrix
        self.outer_solver = sempy.linsolvers.Krylov(
                                            tol = 1e-10,
                                            solver_type = 'cg',
                                            pre = self.M_gl_inv )  
        # Forcing term and linear form
        self.b_uz = self.__rhs()
        # Uzawa operator
        self.U = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__action_uzawa,
                            dtype = 'float' )
        
    def pcg(self,A,b,u,precond,tol=1e-10,max_iter=5000):
        '''
        pcg
        '''
        n = b.size
        p = np.zeros( (n), float)
        s = np.zeros( (n), float)
        r = np.zeros( (n), float)
        z = np.zeros( (n), float)

        s = A*u
        r[:] = b[:] - s[:]
        z = precond*r
        R_new = np.inner(r,z)
        for i in range(max_iter):
            R_old = R_new
            R_new = np.inner(r,z)
            if abs(R_new) < tol*tol :
                flag = i
                #s=A*u#r=b-s
                print 'PCG: Conv at iter.=',i,'sqrt dot(r)=',R_new #,\
                  #'real dot(r)=',np.inner(r,r)
                return [u,flag]
            beta = R_new / R_old
            p[:] = z[:] + beta * p[:]
            s = A*p
            Dd = np.inner( p, s )
            alpha = R_new/Dd
            u[:] = u[:] + alpha * p[:]
            r[:] = r[:] - alpha * s[:]
            #print 'i=',i,'sum=',z.sum()
            z = precond * r
            print 'r.sum()=',r.sum()
     
        if i == max_iter-1:
            flag= i
            print 'pcg() did not converge. dot(r)=',np.inner(r,r)
        
        return [u,flag]
         
    def solve(self):
        '''
        Solves the Stokes problem.
        
        :returns: * **u** - Velocity
                  * **p** - Pressure
                  * **flag** - Gives the number of Uzawa iterations, if 
                    the iterative proceedure is successful.  
        '''
        print 'Solving the Stokes problem...'
        # Step 1: Pressure
        [p_glob, flag] = self.outer_solver.solve( self.U, 
                                                  self.b_uz, 
                                                  self.p.glob() )
        #[p_glob, flag] = self.pcg( self.U, self.b_uz,
        #                           self.p.glob(),
        #                           self.SpaceGL.mass_inv )
        self.p.basis_coeff = self.SpaceGL.mapping_q( p_glob )
        
        # Step 2: Velocity
        for i in range(self.SpaceGL.dim):
            b_temp = self.G[i]*p_glob*(-1.0) + self.b[i]
            [u_temp, flag] = self.inner_solver.solve( 
                                            self.A, 
                                            b_temp, 
                                            self.u[i].glob() )
            self.u[i].basis_coeff = self.Space.mapping_q(u_temp)
        print '...finished.'    
        return self.p, self.u, flag
    
        
    def __rhs(self):
        # The right hand side.
        
        # Solve for the components
        t = np.zeros( (self.Space.dim, self.Space.dof), float)
        for i in range(self.Space.dim):
            [t[i], flag] = self.inner_solver.solve( self.A, 
                                    self.b[i], self.u[i].glob())
        
        # Divergence:
        b = np.zeros( ( self.SpaceGL.dof ), float )
        for i in range( self.Space.dim ):
            b = b + self.D[i] * t[i]
                
        # Compatibility
        if self.SpaceGL.zero == 'yes':
            alpha = b.sum()
            b = b - alpha / self.SpaceGL.dof
        
        return b
        
    
    def __action_uzawa(self,q_in):
        # Action of the Uzawa operator.
        
        # 1: Gradient 
        s = np.zeros( (self.Space.dim, self.Space.dof), float)
        for i in range( self.Space.dim ) :
            s[i] = self.G[i] * q_in
                        
        # 2: Inverse of Helmholtz
        v = np.zeros( (self.Space.dim, self.Space.dof), float)
        t = np.zeros( (self.Space.dim, self.Space.dof), float)
        for i in range( self.Space.dim ) :
            v[i], flag = self.inner_solver.solve( self.A, s[i], t[i] )
        
        # 3: Divergence
        q_out = np.zeros( ( self.SpaceGL.dof ), float )
        for i in range( self.Space.dim ) :
            q_out = q_out + self.D[i] * v[i]
                
        return q_out
            
    
        
            


'''
Sempy: A Python implementation of the spectral element method.
Copyright (C) 2011,  Stian Jensen.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
''' 

import fluid_f90
import fluid_operator_f90
import fluid_def_f90
import sempy
import scipy.sparse.linalg as spl
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from matplotlib import cm

import sempy.operators.operators_f90 as operators_f90
import sempy.space.space_f90 as space_f90

from sempy.precond import precond_f90
import sempy.space.space_f90 as space_f90

import pysparse
from pysparse.direct import umfpack
# Compatibility with older versions
if int(pysparse.__version__[2]) >=2:
    from pysparse.sparse.pysparseMatrix import PysparseMatrix 
    from pysparse import direct
    
if int(pysparse.__version__[2]) <=1:
    from pysparse.pysparseMatrix import PysparseMatrix



import time


class PressurePoisson:
    '''
    The pressure Poisson operator.
    
    .. math::
    
       P = DM^{-1}G
    '''
    def __init__( self, SpaceGL, D, M_inv, G ):
        self.SpaceGL = SpaceGL
        self.D = D
        self.M_inv = M_inv
        self.G = G
        
        P = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__action_poisson,
                            dtype = 'float' )
        return P
        
    
    def __action_poisson(self,q_in):
        #
        if self.SpaceGL.dim == 2:
            # 1
            s_x = self.G[0] * q_in
            s_y = self.G[1] * q_in
            # 2
            v_x = self.M_inv * s_x
            v_y = self.M_inv * s_y
            # 3
            q_out = self.D[0] * v_x + self.D[1] * v_y
            
        return q_out

#class DeflationPrecond:
#    '''
#    Precond.
#    '''
#    def __init__(self,SpaceGL):
#        self.SpaceGL = SpaceGL
#        X_gl_fem = sempy.fluid.SpaceGLFEM( self.SpaceGL )
#        A_fem = sempy.operators.Laplacian( X_gl_fem, 
#                                           assemble = 'yes' ).matrix
#        self.P = sempy.precond.Preconditioner( X_gl_fem, [A_fem] ).matrix
#        #del A_fem
#        self.matrix = spl.LinearOperator( 
#                            ( self.SpaceGL.dof,self.SpaceGL.dof ),
#                              matvec = self.__action__,
#                              dtype = 'float' )
#        
#        
#    def __action__(self,v_in):
#        # MAtrix vector product
#        v = self.P * v_in
#        # Compatibility
#        v_local = self.SpaceGL.mapping_q( v )
#        if self.SpaceGL.dim == 2:
#            v_local = fluid_def_f90.pre_compatibility_2d(v_local)
#        v_out = self.SpaceGL.mapping_qt( v_local )
#        return v_out
#        

def pcg(A,b,u,precond,tol=1e-12,max_iter=5000):
    '''
    pcg
    '''
    n = b.size
    p = np.zeros( (n), float)
    s = np.zeros( (n), float)
    r = np.zeros( (n), float)
    z = np.zeros( (n), float)

    s = A*u
    r[:] = b[:] - s[:]
    z = precond*r
    R_new = np.inner(r,z)
    for i in range(max_iter): 
        R_old = R_new
        R_new = np.inner(r,z)
        if abs(R_new) < tol*tol : 
            flag = i
            #s=A*u#r=b-s
            print 'PCG: Conv at iter.=',i,'sqrt dot(r)=',R_new #,\
                  #'real dot(r)=',np.inner(r,r)
            return [u,flag]
        beta = R_new / R_old
        p[:] = z[:] + beta * p[:]
        s = A*p
        Dd = np.inner( p, s )
        alpha = R_new/Dd
        u[:] = u[:] + alpha * p[:]
        r[:] = r[:] - alpha * s[:]
        #print 'i=',i,'sum=',z.sum()
        z = precond * r
        print 'r.sum()=',r.sum()
     
    if i == max_iter-1: 
        flag= i
        print 'pcg() did not converge. dot(r)=',np.inner(r,r)
        
    return [u,flag] 

 
def cg(A,b,u,tol=1e-10,max_iter=5000):
    '''
    cg
    '''
    n = b.size
    p, s, r = np.zeros((n),np.float), np.zeros((n),np.float), \
              np.zeros((n),np.float)

    s = A * u
    r = b - s
    R_new = np.inner(r,r)
    for i in range(max_iter): 
        R_old = R_new
        R_new = np.inner(r,r)
        if abs(R_new) < tol * tol : 
            flag = i
            print 'CG: Conv. at iter.=',i,'dot(r)=', R_new
            return [u,flag]
        beta = R_new/R_old
        p = r + beta*p
        s = A*p
        Dd = np.inner(p,s)
        alpha = R_new/Dd
        u = u + alpha*p
        r = r - alpha*s
    if i == max_iter-1: 
        flag= i
        print 'cg() did not converge. dot(r)=',np.inner(r,r)
    return [u,flag]        
        
class Deflation:
    '''
    Deflation solver.
    
    Can be applied to solve equations of type 
    
    .. math::
       Sp=b
    
    where 
    
    .. math::
       S=DB^{-1}G
       
    Here, :math:`D` is the divergence operator, :math:`G` is the gradient 
    operator and :math:`B` is either the GLL mass matrix (when used in 
    conjuction with the projection method 
    (:class:`sempy.solvers.ProjectionMethod`)   
    or a Helmholtz operator,  when used in 
    :class:`sempy.solvers.UzawaSolver`. 
    
    One way to solve the above equation is to decompose the solution 
    into coarse and fine constituents:
    
    .. math::
       p=Jp_c + p_f
    
    In short, this leads to two subproblems
    
    .. math:: 
       S_cp_c = J^T(b-Sp_f)\quad (\\text{coarse problem})
       
       S_fp_f = (I-SJS^{-1}_cJ^T)b\quad (\\text{fine problem})
    
    where 
    
    .. math::
       S_c = J^T S J
       
       S_f = (I-SJ S^{-1}_c J^T)S  
    
    The algorithm is as follows
    
      1. Solve the fine problem for :math:`p_f` using preconditioned 
         CG iterations. 
      2. Solve the coarse problem to obtain :math:`p_c`. This is done 
         by a direct method since :math:`S_c\in\mathbb R^{noe\\times noe}`. 
      3. Combine the two solution by the expression 
         :math:`p=Jp_c + p_f`.
          
    In step 1 above, a preconditioner constructed of elemetal 
    Laplacian matrices based on the GL points, with purely homogeneous 
    Neumann conditions, are used.  
               
    :param Space: Discrete space.
    :type Space: :class:`sempy.Space`
    :param SpaceGL: Discrete space.
    :type SpaceGL: :class:`sempy.fluid.SpaceGL`
    :param S: Pressure Poisson operator or the Schur complement. 
    :type S: Scipy LinearOperator
    
    :arguments: * **operator_type** (*string*) - Operator type. Either
                  :literal:`poisson` or :literal:`uzawa`. 
                * **time_step_size** (*float*) - Time step size. Used to
                  create the preconditioner for Uzawa solvers. 
    
    :attributes: * **J** (*Scipy LinearOperator*) - Mapping. 
                           
                   .. math::
                      p=Jp_c
                              
                   where :math:`p\in\mathbb R^{dof}`,
                   :math:`J\in\mathbb R^{dof\\times noe}` and 
                   :math:`p_c\in\mathbb R^{noe}`.  
                              
                 * **JT** (*Scipy LinearOperator*) - The transpose 
                   of :math:`J`.
                   
                   .. math::
                      p_c=J^Tp
                              
                   where :math:`p_c\in\mathbb R^{noe}`,
                   :math:`J^T\in\mathbb R^{noe\\times dof}` and 
                   :math:`p\in\mathbb R^{dof}`.
                     
                 * **S_c** (*Pysparse matrix*)- Coarse operator.
                 * **S_c_inv** (*Pysparse matrix*) - Inverse of the 
                   coarse operator.
                 * **S_f** (*Scipy LinearOperator*) - Fine operator.
                 * **S_f_pre** (*Scipy LinearOperator*) - Preconditioner
                 
    **Usage**::
    
       import sempy 
       
       # Spaces
       X = sempy.Space( filename = filename, n = 10, dim = 2 )
       X_gl = sempy.fluid.SpaceGL( X , zero = 'yes' )

       # Trial functions and time object
       U = sempy.VectorFunction( X )
       p = sempy.Function( X_gl )
       Y = sempy.ode.Time( )

       # Create a projection method instance:
       pro = sempy.solvers.ProjectionMethod( X, X_gl, Y, U, p )
       S = pro.poisson_operator
       
       # An instance of the deflation class:
       deflation = sempy.fluid.Deflation( X, X_gl, S )    
       
    '''
    def __init__( self, Space, SpaceGL, S , 
                  operator_type= 'poisson',
                  time_step_size = 0.1,
                  tol = 1e-10,
                  mu_gl = 'none'):
        print '*** Creating deflation solver..'
        self.Space = Space
        self.SpaceGL = SpaceGL
        self.S = S
        self.operator_type = operator_type
        self.time_step_size = time_step_size 
        self.tol = tol
        # Transport coefficient
        if mu_gl == 'none':
            self.mu_gl = sempy.Function( self.SpaceGL, basis_coeff =  1.0 )
        else:
            self.mu_gl = mu_gl
        # Mapping operator
        self.J = spl.LinearOperator( (self.SpaceGL.dof,self.SpaceGL.noe),
                                      matvec = self.__action_of_j,
                                      dtype = 'float')     
        self.JT = spl.LinearOperator( (self.SpaceGL.noe,self.SpaceGL.dof),
                                      matvec = self.__action_of_jt,
                                      dtype = 'float')     
        self.S_c = self.__coarse_operator()
        # Inverse
        self.LU_coarse = self.__inverse_coarse_operator__()
        self.S_c_inv = spl.LinearOperator( 
                                ( self.SpaceGL.noe, self.SpaceGL.noe ),
                                  matvec = self.__action_coarse_inverse__, 
                                  dtype = 'float' )
        self.S_f = spl.LinearOperator( 
                                ( self.SpaceGL.dof, self.SpaceGL.dof ),
                                  matvec = self.__action_fine__, 
                                  dtype = 'float' )
        
        # Preconditioner
        if self.operator_type == 'poisson':
            self.S_f_pre = self.preconditioner_poisson()
        if self.operator_type == 'uzawa':
            self.S_f_pre = self.preconditioner_uzawa()
            
        self.poisson_solver = sempy.linsolvers.Krylov( 
                                        pre = self.S_f_pre,
                                        tol = self.tol, 
                                        solver_type = 'cg',
                                        maxiter=1000)
        if self.operator_type == 'uzawa':
            self.poisson_solver.maxiter = 300
        print '*** Finished creating deflation solver.'
        
    
    def preconditioner_poisson(self):
        '''
        Preconditioner for the fine problem when 
        :math:`S=DM^{-1}G`.
        
        :returns: A Scipy LinearOperator. 
        '''
        X_gl_fem = sempy.fluid.SpaceGLFEM( self.SpaceGL )
        A_fem = sempy.operators.Laplacian( X_gl_fem, 
                                           assemble = 'yes' ).matrix
        self.P = sempy.precond.Preconditioner( X_gl_fem, 
                                               [A_fem],
                                               #library = 'superlu' ).matrix
                                               library = 'umfpack' ).matrix
        del A_fem
        pre_matrix = spl.LinearOperator( 
                            ( self.SpaceGL.dof,self.SpaceGL.dof ),
                              matvec = self.__action_precond__,
                              dtype = 'float' )
        return pre_matrix
    
    def preconditioner_uzawa(self):
        '''
        Preconditioner for the fine problem when 
        :math:`S=DH^{-1}G`.
        
        :returns: A LinearOperator. 
        '''
        X_gl_fem = sempy.fluid.SpaceGLFEM( self.SpaceGL )
    
        # Laplacian
        A_fem = sempy.operators.Laplacian( X_gl_fem, 
                                           assemble = 'yes' ).matrix
        #
        self.P_1 = sempy.precond.Preconditioner( 
                            X_gl_fem, 
                            [A_fem], 
                            library = 'umfpack' ).matrix
        #
        coeff = self.mu_gl.glob()
        self.P_2 = sempy.precond.Preconditioner( 
                            self.SpaceGL, 
                            [self.SpaceGL.mass_matrix],
                            #scaling_factor= [self.time_step_size*coeff[0]],
                            library = 'superlu' ).matrix
        #self.P = sempy.operators.MultipleOperators([P_2],#[P_1,P_2], 
        #                                           assemble = 'no'
        #                                           ).matrix
        
        del A_fem
        pre_matrix = spl.LinearOperator( 
                            ( self.SpaceGL.dof,self.SpaceGL.dof ),
                              matvec = self.__action_precond_uzawa__,
                              dtype = 'float' )
        return pre_matrix
    
    def __action_precond_uzawa__(self,v_in):
        co = self.mu_gl.glob()
        coeff = co[0]
        # Matrix vector product
        v_1 = self.P_1 * v_in
        v_2 = coeff*self.time_step_size * (self.P_2 * v_in)
        v = v_1 +v_2
        # Compatibility
        v_out = self.compatibility( v )
        return v_out
        
    def __action_precond__(self,v_in):
        # Matrix vector product
        v = self.P * v_in
        # Compatibility
        v_out = self.compatibility( v )
        return v_out
    
        
    def __solve_coarse__( self, b, phi_f ):
        # rhs
        b_rhs = self.JT * ( b - ( self.S * phi_f ) )
        # Compatibility condition
        #if self.SpaceGL.zero == 'yes':
        #    b_sum = b_rhs.sum()
        #    b_rhs = b_rhs - b_sum / np.float( self.SpaceGL.noe )
        #print 'coarse sum rhs=',b_rhs.sum()
        phi_c = self.S_c_inv * b_rhs
        return phi_c
    
    def solve_standard( self, b ,x ):
        '''
        Solve the pressure Poisson equation with standard CG iterations, 
        i.e. non-preconditioned.
        
        .. math::
           Sx=b
        
        This method can be used to compare the efficiency of the 
        preconditioned deflation solver. 
        
        :param b: Right hand side. For projection methods: 
                  b=D*u_hat 
        :type b: array
        :param x: Solution vector.
        :type x: array
        :returns: Solution vector and a flag.
        
        **Example**::
        
           # To solve S*phi = b:
           [phi, flag] = deflation.solve_standard( b, phi )
           
        '''
        t1 = time.clock()
        self.poisson_solver.pre = 'none'
        [ phi, flag ] = self.poisson_solver.solve( self.S, b, x )
        t2 = time.clock()
        print 'time standard=',t2-t1
        return [phi, flag]
    
    def compatibility( self, v ):
        '''
        A method to enforce a local compatibility condition when solving 
        the fine problem in the deflation solver. This is necessary since the 
        preconditioner is blockwise semi-definite. 
        '''
        v_local = self.SpaceGL.mapping_q( v )
        
        if self.SpaceGL.dim == 1:
            print 'Deflation: compatibility() not implemented for 1D.'
        
        if self.SpaceGL.dim == 2:
            v_local = fluid_def_f90.pre_compatibility_2d(v_local)
            
        if self.SpaceGL.dim == 3:
            v_local = fluid_def_f90.pre_compatibility_3d(v_local)
                
        v_out = self.SpaceGL.mapping_qt( v_local )
        
        return v_out
                
    def solve( self, b, x ):
        '''
        Solve the pressure Poisson equation with the deflation method.
        
        :param b: Right hand side. For projection methods: 
                  b=D*u_hat 
        :type b: array
        :param x: Solution vector.
        :type x: array
        :returns: Solution vector and a flag.'
        
        **Example**::
        
           # To solve S*phi = b:
           [phi, flag] = deflation.solve( b, phi )
           
        '''
        #t1 = time.clock()
        
        # Right hand side: b = D * u
        s1 = self.S * ( self.J * ( self.S_c_inv * (self.JT * b ) ) )
        b_rhs = np.zeros( ( self.SpaceGL.dof ), float )
        b_rhs[:] = b[:] - s1[:]
        b_rhs = self.compatibility( b_rhs ) 
        # Solve fine problem
        [ phi_f, flag ] = self.poisson_solver.solve( self.S_f, b_rhs, x )
        #[ phi_f, flag ] = self.pcg( self.S_f, b_rhs, x, Pre )
        #[ phi_f, flag ] = cg( self.S_f, b_rhs, x )
        #flag = 0
        #phi_f = self.S_f_pre * b_rhs
        
        # Solve coarse problem
        phi_c = self.__solve_coarse__( b, phi_f )
        
        # Combine
        phi = self.J * phi_c + phi_f
        # Test rhs
        #b_local = self.SpaceGL.mapping_q(b_rhs)
        #for k in range(self.SpaceGL.noe):
        #    alpha = 0.0
        #    for m in range(self.SpaceGL.n):
        #        for n in range(self.SpaceGL.n):
        #            alpha = alpha + b_local[k,m,n]
        #    print 'k=',k,'alpha=',alpha
        # Test phi_f
        #b_local = self.SpaceGL.mapping_q( phi_f )
        #for k in range(self.SpaceGL.noe):
        #    alpha = 0.0
        #    for m in range(self.SpaceGL.n):
        #        for n in range(self.SpaceGL.n):
        #            alpha = alpha + b_local[k,m,n]
        #    print 'k=',k,'phi alpha=',alpha
        #t2 = time.clock()
        #print 'time def=',t2-t1
        
        return [phi, flag]
    
        
    def __inverse_coarse_operator__(self):
        if pysparse.__version__[2] >='2':
            #LU_coarse = direct.superlu.factorize( self.S_c.to_csr() )
            LU_coarse = umfpack.factorize( self.S_c )
        if pysparse.__version__[2] <='1':
            #LU_coarse = pysparse.superlu.factorize( self.S_c.to_csr() )
            LU_coarse = umfpack.factorize( self.S_c )
        return LU_coarse
        
        
    def __action_coarse_inverse__(self,v):
        # v is the RHS: A*w=v
        w = np.zeros(self.SpaceGL.noe, float)
        self.LU_coarse.solve(v,w)
        #print 'coarse='
        #print w
        #if self.SpaceGL.zero == 'yes':
        #    #print '--> zero'
        #    w.sum()
        #    w=w-w.sum()/float(self.SpaceGL.noe)
            #print 'wsum=',w.sum()
        return w
    
    
    def __action_fine__(self,q):
        # 
        s1 = self.S * q
        s2 = self.S * ( self.J * ( self.S_c_inv* ( self.JT*s1 ) ) )
        s = s1 -s2
        #print 'obbs __action_fine__ compatibility()'
        s = self.compatibility( s )
        return s
        
    
    def __coarse_operator(self):
        #
        noe = self.SpaceGL.noe
        n = self.SpaceGL.n
        A = pysparse.spmatrix.ll_mat( noe, noe )
        
        if self.SpaceGL.dim == 1:
            v = np.zeros( ( noe, n ), float )
            for k in range( noe ):
                v[:,:]=0.0
                v[k,:]=1.0
                v_glob = self.SpaceGL.local_to_global( v ) 
                w = self.S * v_glob
                Ad = self.JT * w
                for i in range(noe):
                    A[i,k] = Ad[i]
                    
        if self.SpaceGL.dim == 2:
            v = np.zeros( ( noe, n, n ), float )
            for k in range( noe ):
                v[:,:,:]=0.0
                v[k,:,:]=1.0
                v_glob = self.SpaceGL.local_to_global( v ) 
                w = self.S * v_glob
                Ad = self.JT * w
                for i in range(noe):
                    A[i,k] = Ad[i]
                    
        if self.SpaceGL.dim == 3:
            v = np.zeros( ( noe, n, n, n ), float )
            for k in range( noe ):
                v[:,:,:,:]=0.0
                v[k,:,:,:]=1.0
                v_glob = self.SpaceGL.local_to_global( v ) 
                w = self.S * v_glob
                Ad = self.JT * w
                for i in range(noe):
                    A[i,k] = Ad[i]
        #print 'A='
        #print A
        # Check if singular
        s = np.zeros((noe),float)
        for i in range(noe):
            for j in range(noe):
                s[i] = s[i] + A[i,j]
        
        if abs(s.sum()) <= 1e-10:
            print '-> Coarse pressure matrix is singular:',s.sum()
        
        return A
            
    def __action_of_jt(self,v):
        #
        v_loc = self.SpaceGL.mapping_q( v )
        if self.SpaceGL.dim == 1:
            w = fluid_def_f90.action_f_jt_1d(v_loc) 
        if self.SpaceGL.dim == 2:
            w = fluid_def_f90.action_f_jt_2d(v_loc) 
        if self.SpaceGL.dim == 3:
            w = fluid_def_f90.action_f_jt_3d(v_loc) 
        return w
        
        
    def __action_of_j(self,v):
        #
        if self.SpaceGL.dim == 1:
            w = fluid_def_f90.action_f_j_1d( v, self.SpaceGL.n ) 
         
        if self.SpaceGL.dim == 2:
            w = fluid_def_f90.action_f_j_2d( v, self.SpaceGL.n ) 
        
        if self.SpaceGL.dim == 3:
            w = fluid_def_f90.action_f_j_3d( v, self.SpaceGL.n )
        
        w_out = self.SpaceGL.local_to_global(w)
        return w_out 
            
        
class SpaceGL:
    '''
    A space for pressure.
     
    :argument Space: Discrete space.
    :type Space: :class:`sempy.Space`
    
    :attributes: * **n** (*int*)- Number of nodal points in each direction 
                   of an element. 
                 * **dim** (*int*) - Dimension :math:`d` of the computational 
                   domain :math:`\Omega\in\mathbb R^d`.  
                 * **zero** (*string*) - Determines the type of space. Either 
                   :literal:`yes`, meaning
                   
                   .. math::
                   
                      \\int_\Omega p\,d\Omega=0
                   
                   or :literal:`no` (default). 
                 * **noe** (*int*)- Number of spectral elements.  
                 * **dof** (*int*)- Degrees of freedom of the GL mesh. 
                 * **points** (*array*)- GL quadrature points 
                   :math:`\\underline{\\xi}\\in\\mathbb R^{N-1},\\ -1< \\xi_j < 1`.
                 * **weights** (*array*)- GL quadrature weights 
                   :math:`\\underline{\\tilde\\rho}\\in\\mathbb R^{N+1}` 
                   corresponding to the points :math:`\\underline{\\tilde\\xi}`. 
                   Used to evaluate an integral over the reference domain, e.g.:
                 
                   .. math::
                      \\hat\\Omega\\in\\mathbb R^1: \\int_{\\hat\\Omega} f d\\hat\\Omega\\approx
                      \\sum_i\\tilde\\rho_if_i
                      
                      \\hat\\Omega\\in\\mathbb R^2: \\int_{\\hat\\Omega} f d\\hat\\Omega\\approx
                      \\sum_i\\sum_j\\tilde\\rho_i\\tilde\\rho_jf_{ij}
                 
                 * **x,y,z** (*array*)- Coordinate arrays (given in local 
                   data distribution). 
                 * **I_tilde** (*array*)- Interpolation matrix. 
                   Used to interpolate a function from GLL points to GL
                   points.
                 
                   .. math::
                      
                      w_{i}=\\sum_{m=0}^{N}\\tilde I_{im}u_{m}
                      
                 * **I_tilde_gll** (*array*)- Interpolation matrix. Used to 
                   interpolate a function from GL points to GLL points.
                 * **D_tilde** (array)- Derivative matrix (numpy array). 
                   Used to interpolate derivatives from GLL to GL.
                   
                   .. math::
                   
                      \\frac{\partial u}{\partial\\xi}\\bigg|_{\\alpha\\beta}=
                      \sum_{m=0}^N\sum_{n=0}^N\\frac{\partial\ell_m}{\partial\\xi}
                      (\\xi_\\alpha)\ell_n(\\eta_\\beta)u_{mn}=
                      \sum_{m=0}^N\sum_{n=0}^N\\tilde D_{\\alpha m}
                        \\tilde I_{\\beta n}u_{mn}
                      
                   here :math:`(\\alpha,\\beta)` denotes the GL points. 
                   
                 * **theta** (*array*)- Connectivity array.   
                 * **mass_matrix** (*Pysparse matrix*) - GL mass matrix.
                 * **mass_inv** (*Scipy LinearOperator*) - Inverse of the GL 
                   mass matrix.  
                      
    **Example**::
    
       import sempy
       
       X = sempy.Space( filename = 'square', n = 10, dim = 2 )
       
       X_gl = sempy.fluid.SpaceGL( X )
       
       X_gl.plot_mesh()
                   
    '''
    def __init__(self,Space, zero = 'no'):
        self.Space = Space
        self.n   = self.Space.n - 2 
        self.noe = self.Space.noe
        self.dim = self.Space.dim
        self.type = 'GL'
        self.zero = zero
        self.noe_bc = 0
        self.basic = sempy.Basic()
        
        if self.Space.n <= 3:
            print 'Polynomial degree to low to create interpoaltion matrix'
            print 'Remedy: increase the polynomial degree to >=3.'
        else:
            self.points, self.weights = self.basic.gauss_legendre(self.n)
            self.I_tilde = self.__interpolation_matrix()
            self.I_tilde_gll = self.__interpolation_matrix_gll()
            self.__coordinates()
            self.D_tilde = self.__derivative_matrix_tilde()
            # Connectivity
            self.theta = self.__connectivity()
            # DOF
            if self.dim == 1:
                self.dof = self.n*self.noe
            if self.dim == 2:
                self.dof = (self.n*self.n)*self.noe
            if self.dim == 3:
                self.dof = (self.n*self.n*self.n)*self.noe
            # The GL mas matrix
            self.mass_matrix = self.__mass_matrix__()
            # inverse of mass matrix gl
            self.mass_inv = self.__mass_matrix_inv__( self.mass_matrix )
           
            #self.mass_inv = spl.LinearOperator( 
            #              ( self.dof,self.dof ),
            #                matvec = self.__action_mass_matrix_inv,
            #                dtype = 'float' )
        
        self.fem = SpaceGLFEM( self )
    
    def __mass_matrix_inv__( self, M ):
        M_inv = PysparseMatrix( size = self.dof )
        #M_inv = pysparse.spmatrix.ll_mat( self.dof, self.dof )
        
        for i in range( self.dof ):
            M_inv[i,i] = 1.0/M[i,i]
            
        return M_inv
    
        
    def __mass_matrix__( self ):
        #M = pysparse.spmatrix.ll_mat( self.dof, self.dof )
        M = PysparseMatrix( size = self.dof )
        
        if self.dim == 1:
            mass_local = np.zeros( ( self.noe, self.n ), float )
            for k in range(self.noe):
                for m in range(self.n):
                        mass_local[k,m] = self.jac[k,m]*\
                                          self.weights[m]
                                           
        if self.dim == 2:
            mass_local = np.zeros( (self.noe,self.n,self.n), float )
            for k in range(self.noe):
                for m in range(self.n):
                    for n in range(self.n):
                        mass_local[k,m,n] = self.jac[k,m,n]*\
                                            self.weights[m]*\
                                            self.weights[n]
        
        if self.dim == 3:
            mass_local = np.zeros( (self.noe,self.n,self.n,self.n), float )
            for k in range(self.noe):
                for m in range(self.n):
                    for n in range(self.n):
                        for o in range(self.n):
                            mass_local[k,m,n,o] = self.jac[k,m,n,o]*\
                                                  self.weights[m]*\
                                                  self.weights[n]*\
                                                  self.weights[o]
                                               
        mass_global = self.local_to_global( mass_local )
        for i in range(self.dof):
            M[i,i] = mass_global[i]
        return M
            
    def __action_mass_matrix_inv(self,u_in):
        u = self.mapping_q(u_in)
        if self.dim == 1:
            b = fluid_operator_f90.action_mass_matrix_gl_inv_1d(u,
                                                    self.weights,self.jac)
            
        if self.dim == 2:
            b = fluid_operator_f90.action_mass_matrix_gl_inv_2d(u,
                                                    self.weights,self.jac)
            
        if self.dim == 3:
            b = fluid_operator_f90.action_mass_matrix_gl_inv_3d(u,
                                                    self.weights,self.jac)
        u_out = self.mapping_qt(b)    
        
        return u_out
        
        
    def __connectivity(self):
        '''
        Connectivity array.
        '''
        if self.dim == 1:
            i = 0
            theta = np.zeros((self.noe,self.n),int)
            for k in range(self.noe):
                for m in range(self.n):
                    theta[k,m] = i
                    i = i + 1
        if self.dim == 2:
            i = 0
            theta = np.zeros((self.noe,self.n,self.n),int)
            for k in range(self.noe):
                for m in range(self.n):
                    for n in range(self.n):
                        theta[k,m,n] = i
                        i = i + 1
        if self.dim == 3:
            i = 0
            theta = np.zeros((self.noe,self.n,self.n,self.n),int)
            for k in range(self.noe):
                for m in range(self.n):
                    for n in range(self.n):
                        for o in range(self.n):
                            theta[k,m,n,o] = i
                            i = i + 1
                        
        return theta
            
    def gradient_vector(self,u):
        '''
        Gradient vector.
        
        :param u: Basis coefficients in local data structure.
        '''
        if self.Space.dim == 1:
            u_gll = self.interpolation_gll( u )
            u_x = self.Space.gradient_vector( u_gll )
            u_x_gl = self.interpolation( u_x )
            return u_x_gl
        
        if self.Space.dim == 2:
            u_gll = self.interpolation_gll( u )
            u_x, u_y = self.Space.gradient_vector( u_gll )
            u_x_gl = self.interpolation( u_x )
            u_y_gl = self.interpolation( u_y )
            return u_x_gl, u_y_gl
        
        if self.Space.dim == 3:
            u_gll = self.interpolation_gll( u )
            u_x, u_y, u_z = self.Space.gradient_vector( u_gll )
            u_x_gl = self.interpolation( u_x )
            u_y_gl = self.interpolation( u_y )
            u_z_gl = self.interpolation( u_z )
            return u_x_gl, u_y_gl, u_z_gl
        
    def quadrature(self,f):
        """Computes the value of the integral 
        
           .. math::
              \\kappa = \\int_\\Omega f\\,d\\Omega 
           
           for :math:`f\\in L^2(\\Omega)` with GL quadrature rules. 
        
           :param f: Basis coefficients of the function to be integrated given in local
                     data representation (since f in L2). 
           :returns: Value :math:`\\kappa` of the integral.
        
        """
        if self.dim ==1:
            kappa = space_f90.quadrature_1d(self.weights,self.jac,f)
        if self.dim ==2:
            kappa = space_f90.quadrature_2d(self.weights,self.jac,f)
        if self.dim ==3:
            kappa = space_f90.quadrature_3d(self.weights,self.jac,f)
        return kappa
    
    def l2_norm(self,u):
        """Computes the :math:`L^2` - norm
           
           .. math::
              ||u||_{L^2(\\Omega)}=\\bigg(\\int_{\\Omega} u^2d\\Omega\\bigg)^{1/2}
              
           using GL quadrature. 
           
           :param u: Basis coefficients in local data structure.
           :returns: The :math:`L^2` - norm as a floating number.
           
           Usage::
           
             u  = np.ones((mesh.dof),float)
             print 'L2-norm=',mesh.l2norm(u)

        """
        if self.dim ==1:
            norm = space_f90.l2norm_1d(self.weights,self.jac,u)
        if self.dim ==2:
            norm = space_f90.l2norm_2d(self.weights,self.jac,u)
        if self.dim ==3:
            norm = space_f90.l2norm_3d(self.weights,self.jac,u)
            
        return norm
    
    def local_to_global(self,u_loc):
        '''
        Maps from local to global. Identical to 
        :class:`SpaceGL.mapping_qt`. This method only exist to make the 
        present class compatible with the :class:`sempy.Function` class.  
                
        :param array u_loc: Basis coefficients in local data 
                            representation.
        :returns: (*array*) - The mapped basis coefficients in global data
                  representation. 
                             
        '''
        if self.dim == 1:
            #u_glob = np.zeros((self.dof),float)
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        i = self.theta[k,m]
            #        u_glob[i] = u_loc[k,m]
            u_glob = space_f90.local_to_global_1d(u_loc,self.theta,self.dof)
            return u_glob
        
        if self.dim == 2:
            #u_glob = np.zeros((self.dof),float)
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        for n in range(self.n):
            #            i = self.theta[k,m,n]
            #            u_glob[i] = u_loc[k,m,n]
            u_glob = space_f90.local_to_global_2d(u_loc,self.theta,self.dof)
            return u_glob
        
        if self.dim == 3:
            u_glob = space_f90.local_to_global_3d(u_loc,self.theta,self.dof)
            return u_glob
        
    def mapping_qt(self,u_loc):
        '''
        Maps from local to global
        
        :param array u_loc: Basis coefficients in local data representation.
        :returns: (*array*) - The mapped basis coefficients in global data
                  representation. 
        
        '''
        if self.dim == 1:
            #u_glob = np.zeros((self.dof),float)
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        i = self.theta[k,m]
            #        u_glob[i] = u_loc[k,m]
            u_glob = space_f90.local_to_global_1d(u_loc,self.theta,self.dof)
            return u_glob
        
        if self.dim == 2:
            #u_glob = np.zeros((self.dof),float)
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        for n in range(self.n):
            #            i = self.theta[k,m,n]
            #            u_glob[i] = u_loc[k,m,n]
            u_glob = space_f90.local_to_global_2d(u_loc,self.theta,self.dof)
            return u_glob
        
        if self.dim == 3:
            #u_glob = np.zeros((self.dof),float)
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        for n in range(self.n):
            #            for o in range(self.n):
            #                i = self.theta[k,m,n,o]
            #                u_glob[i] = u_loc[k,m,n,o]
            u_glob = space_f90.local_to_global_3d(u_loc,self.theta,self.dof)
            return u_glob
    
    def mapping_q(self,u_glob):
        '''
        Maps from global to local
        
        :param array u_glob: Basis coefficients in global data 
                             representation.
        :returns: (*array*) - The mapped basis coefficients in local data
                  representation. 
        
        '''
        if self.dim == 1:
            #u_loc = np.zeros( (self.noe,self.n), float )
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        i = self.theta[k,m]
            #        u_loc[k,m] = u_glob[i]
            u_loc = space_f90.mapping_q_1d(u_glob,self.theta)
            return u_loc
        
        if self.dim == 2:
            #u_loc = np.zeros( (self.noe,self.n,self.n), float )
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        for n in range(self.n):
            #            i = self.theta[k,m,n]
            #            u_loc[k,m,n] = u_glob[i]
            u_loc = space_f90.mapping_q_2d(u_glob,self.theta)
            return u_loc
        
        if self.dim == 3:
            #u_loc = np.zeros( (self.noe,self.n,self.n,self.n), float )
            #for k in range(self.noe):
            #    for m in range(self.n):
            #        for n in range(self.n):
            #            for o in range(self.n):
            #                i = self.theta[k,m,n,o]
            #                u_loc[k,m,n,o] = u_glob[i]
            u_loc = space_f90.mapping_q_3d(u_glob,self.theta)
            return u_loc
        
    def plot_mesh(self):
        '''
        Plots the GL mesh.
        '''
        if self.dim == 1:
            u = np.zeros((self.noe,self.n),float)
            for k in range(self.noe):
                plt.plot(self.x[k,:],u[k,:],'o-')
            # test interpolate
            #x_gll=self.interpolation_gll(self.x)
            #u = np.zeros((self.noe,self.n+2),float)
            #for k in range(self.noe):
            #    plt.plot(x_gll[k,:],u[k,:],'o-')
            plt.show()
            
        if self.dim == 2:
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            u = np.zeros((self.noe,self.n,self.n),float)
            #u = self.x
            #u_glob = self.mapping_qt(self.x)
            #u = self.mapping_q(u_glob)
            for k in range(self.noe):
                ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
            # test interpoalte
            #x_gll=self.interpolation_gll(self.x)
            #y_gll=self.interpolation_gll(self.y)
            #u = np.zeros((self.noe,self.n+2,self.n+2),float)
            #for k in range(self.noe):
            #    ax.plot_wireframe(x_gll[k,:,:],y_gll[k,:,:],u[k,:,:])
            
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            plt.show()
        if self.dim == 3:
            #try:
            from enthought.mayavi import mlab
            x = self.x#interpolation_gll(self.x)
            y = self.y#interpolation_gll(self.y)
            z = self.z#interpolation_gll(self.z)
            u = z#np.zeros((self.noe,self.n,self.n,self.n),float)
        
            #s = mlab.mesh(x, y, z)
            #levels = np.arange(0, 1, 0.1)
            levels=[0.2,0.4,0.6,0.8]
            for k in range(self.noe):
                #s=mlab.contour3d( x[k,:,:,:], y[k,:,:,:], 
                #                  z[k,:,:,:], u[k,:,:,:],
                #                  contours=levels)
                s=mlab.points3d( x[k,:,:,:], y[k,:,:,:], 
                               z[k,:,:,:], u[k,:,:,:])
            mlab.show()
            #except:
                    
    def __coordinates(self):
        '''
        '''
        if self.dim == 1:
            self.x = self.interpolation(self.Space.x)
            self.y = 0.0 
            self.z = 0.0
            self.jac = self.interpolation(self.Space.jac)
            
        if self.dim == 2:
            self.x = self.interpolation(self.Space.x)
            self.y = self.interpolation(self.Space.y)
            self.z = 0.0
            self.jac = self.interpolation(self.Space.jac)
            
        if self.dim == 3:
            self.x = self.interpolation(self.Space.x)
            self.y = self.interpolation(self.Space.y)
            self.z = self.interpolation(self.Space.z)
            self.jac = self.interpolation(self.Space.jac)
        
        return    
            
        
    def interpolation(self,u_gll):
        '''
        Interpolation from GLL to GL.
        
        :param array u_gll: Basis coefficients of a GLL function given in 
                            local data representation.
        :returns: (*array*) - Basis coefficients of a GL function given in 
                  local data representation.
        '''
        if self.dim == 1:
            u_gl = fluid_f90.interpolation_1d( u_gll, self.I_tilde )
        if self.dim == 2:
            u_gl = fluid_f90.interpolation_2d( u_gll, self.I_tilde )
        if self.dim == 3:
            u_gl = fluid_f90.interpolation_3d( u_gll, self.I_tilde )
            
        return u_gl
    
    def interpolation_gll(self,u_gl):
        '''
        Interpolation from GL to GLL.
        
        :param array u_gl: Basis coefficients of a GL function given in 
                           local data representation.
        :returns: (*array*) - Basis coefficients of a GLL function given in 
                  local data representation.
                            
        '''
        if self.dim == 1:
            u_gll = fluid_f90.interpolation_gll_1d( u_gl, self.I_tilde_gll )
        if self.dim == 2:
            u_gll = fluid_f90.interpolation_gll_2d( u_gl, self.I_tilde_gll )
        if self.dim == 3:
            u_gll = fluid_f90.interpolation_gll_3d( u_gl, self.I_tilde_gll )
            
        return u_gll
                    
    def __interpolation_matrix(self):
        '''
        The interpolation matrix (GLL to GL)
        '''
        gll = self.Space.points # GLL quadrature points
        gl  = self.points # GL quadrature points
        I_tilde = fluid_f90.interpolation_matrix( gll, gl )
        return I_tilde
    
    def __derivative_matrix_tilde(self):
        '''
        The interpolation matrix (GLL to GL)
        '''
        #gll = self.Space.points # GLL quadrature points
        #gl  = self.points # GL quadrature points
        D_tilde = fluid_f90.derivative_matrix_tilde(self.I_tilde,self.Space.D)
        return D_tilde
    
    def __interpolation_matrix_gll(self):
        '''
        The interpolation matrix (GL to GLL)
        '''
        gll = self.Space.points # GLL quadrature points
        gl  = self.points # GL quadrature points
        I_tilde_gll = fluid_f90.interpolation_matrix_gll(gll,gl)
        return I_tilde_gll           


class SpaceGLFEM:
    '''
    Finite element space based on the GL points.
    
    :param SpaceGL: Discrete GL space.
    :type SpaceGL: :class:`sempy.fluid.SpaceGL` 
    '''
    def __init__(self,SpaceGL):
        self.SpaceGL = SpaceGL
        self.type    = 'GL'
        self.dim     = self.SpaceGL.dim
        self.n       = 2
        self.dof     = self.SpaceGL.dof
        self.noe_bc = 0
        self.basic = sempy.Basic()
        
        
        # Number of elements
        __n = self.SpaceGL.n-1
        __noe =self.SpaceGL.noe
        
        if self.dim == 1:
            self.noe = ( __n ) * __noe
            
        if self.dim == 2:
            self.noe = ( __n ) * ( __n ) * __noe
        
        if self.dim == 3:
            self.noe = ( __n ) * ( __n ) * ( __n ) * __noe
            #self.theta_bc = 
        # GLL points and weigts
        #self.points, self.weights = sempy.basic.gauss_legendre(self.n)
        self.points, self.weights = \
                      self.basic.gauss_lobatto_legendre( self.n )
        #print 'SpaceGLFEM p=',self.points
        #print 'SpaceGLFEM w=',self.weights
        #self.x, self.y, self.theta = 
        self.__fem_coor()
        self.D = self.basic.derivative_matrix_gll( self.n )
        #print 'D='
        #print self.D
        self.jac = self.__jacobian()
        
        #self.I_tilde = self.__interpolation_matrix()
        #self.I_tilde_gll = self.__interpolation_matrix_gll()
        #self.__coordinates()
        #self.D_tilde = self.__derivative_matrix_tilde()
            
        
        #self.D = basic.lagrange_derivative_matrix_gll(self.n)
        #self.points, self.weights = basic.gauss_lobatto_legendre(self.n)
    def mask(self,v):
        return v
        
    def __jacobian(self):
        # The jacobian.
        if self.dim == 1:
            jac = sempy.space.space_f90.geometric_1d( self.x, self.D )
        if self.dim == 2:
            jac = sempy.space.space_f90.geometric_2d( self.x, self.y, 
                                                      self.D )
        if self.dim == 3:
            jac = sempy.space.space_f90.geometric_3d( self.x, self.y, 
                                                      self.z, self.D )
        return jac
            
    def sem_to_fem(self,u_sem):
        '''
        A mapping from the SEM space to FEM space. 
        '''
        if self.dim == 1:
            u_fem = precond_f90.sem_to_fem_1d( u_sem, self.n, self.noe)
        if self.dim == 2:
            u_fem = precond_f90.sem_to_fem_2d( u_sem, self.n, self.noe)
        if self.dim == 3:
            u_fem = precond_f90.sem_to_fem_3d( u_sem, self.n, self.noe)
            
        return u_fem
        
    def local_to_global(self,w):
        if self.dim == 1:
            v = space_f90.local_to_global_1d(w,self.theta,self.dof)
        if self.dim == 2:
            v = space_f90.local_to_global_2d(w,self.theta,self.dof)
        if self.dim == 3:
            v = space_f90.local_to_global_3d(w,self.theta,self.dof)
        return v
    
    def __fem_coor(self):
        # Coordinates
        if self.dim == 1:
            self.x, self.theta = precond_f90.fem_coor_1d( 
                                               self.SpaceGL.x,
                                               self.SpaceGL.theta, 
                                               self.n,
                                               self.noe )
            return self.x, self.theta
        
        if self.dim == 2:
            self.x,self.y,self.theta = precond_f90.fem_coor_2d( 
                                               self.SpaceGL.x,
                                               self.SpaceGL.y,
                                               self.SpaceGL.theta, 
                                               self.n,
                                               self.noe )
            return self.x, self.y, self.theta
        
        if self.dim == 3:
            self.x, self.y, self.z, self.theta = \
                        precond_f90.fem_coor_3d( self.SpaceGL.x,
                                                 self.SpaceGL.y,
                                                 self.SpaceGL.z,
                                                 self.SpaceGL.theta, 
                                                 self.n,self.noe )
            return self.x, self.y, self.z, self.theta
    
    def plot_mesh(self):
        '''
        Plots the numerical mesh.
        ''' 
        if self.dim == 2:
            u = np.zeros((self.noe,self.n,self.n),float)
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            for k in range(self.noe):
                ax.plot_wireframe(self.x[k,:,:],self.y[k,:,:],u[k,:,:])
            plt.show()
    
        

class Gradient:
    '''
    The gradient operator 
    
    .. math::
    
       \underline{\mathbf G} =[\underline{G}_x,
                   \underline{G}_y,\underline{G}_z]^T 
        
    as found in the Stokes- and Navier-Stokes equations. 
    
    .. math::
    
       \\nabla p \\Rightarrow 
       \underbrace{-\int_\Omega p\\nabla\cdot \mathbf w\,d\Omega}_{\\text{GL quadrature}}
       \Rightarrow b_N(p,\mathbf w) 
       
       b_N(p,\mathbf w)\\Rightarrow 
       \underline{\mathbf G}\,\underline{p}
    
    :argument Space: Discrete space.
    :type Space: :class:`sempy.Space`
    :argument SpaceGL: Discrete GL space.
    :type SpaceGL: :class:`sempy.fluid.SpaceGL`
        
    :attributes: * **matrix_x**, **matrix_y**, **matrix_z** 
                   (*Scipy LinearOperator*) - The components 
                   of the gradient operator
                   :math:`\underline{\mathbf G} =[\underline{G}_x\,\underline{G}_y\, \underline{G}_z]^T`. 
                 * **matrix** (*List of Scipy LinearOperators*) - The gradient 
                   operator :math:`\underline{\mathbf G}`.   
    
    **Example**::
    
       import sempy

       X = sempy.Space( filename = 'cube', n = 4, dim = 3 )
       X_gl=sempy.fluid.SpaceGL( X )
    
       grad = sempy.fluid.Gradient( X, X_gl )
    
       G_x = grad.matrix_x
       G_z = grad.matrix_y
       G_y = grad.matrix_z
    
    '''
    def __init__(self,Space,SpaceGL):#,assemble='no'):
        self.Space = Space
        self.SpaceGL = SpaceGL
        #self.assemble = assemble
        
        if self.Space.dim == 1:
            self.matrix_x = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_1d,
                                     dtype = 'float' )
            self.matrix = [ self.matrix_x ]
            
        if self.Space.dim == 2:
            g11,g12,g21,g22=\
               operators_f90.geometric_convection_2d(self.Space.x,
                                                     self.Space.y,
                                                     self.Space.D) 
            # Interpolate to GL mesh
            self.g11 = self.SpaceGL.interpolation(g11)
            self.g12 = self.SpaceGL.interpolation(g12)
            self.g21 = self.SpaceGL.interpolation(g21)
            self.g22 = self.SpaceGL.interpolation(g22)
            
            # Matrix
            self.matrix_x = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_x_2d,
                                     dtype = 'float' )    
            self.matrix_y = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_y_2d,
                                     dtype = 'float' )
            self.matrix = [ self.matrix_x, self.matrix_y ]
        if self.Space.dim == 3:
            self.__geometric_3d()
            # Matrix
            self.matrix_x = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_x_3d,
                                     dtype = 'float' ) 
            # Matrix
            self.matrix_y = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_y_3d,
                                     dtype = 'float' ) 
            # Matrix
            self.matrix_z = spl.LinearOperator( 
                                   ( self.Space.dof,self.SpaceGL.dof ),
                                     matvec = self.__action_z_3d,
                                     dtype = 'float' ) 
            #
            self.matrix = [ self.matrix_x, self.matrix_y, self.matrix_z ]
            
              
    def __geometric_3d(self):
        # geometric 
        noe = self.Space.noe
        n = self.Space.n
        g11 = np.zeros((noe,n,n,n),float)
        g12 = np.zeros((noe,n,n,n),float)
        g13 = np.zeros((noe,n,n,n),float)
        g21 = np.zeros((noe,n,n,n),float)
        g22 = np.zeros((noe,n,n,n),float)
        g23 = np.zeros((noe,n,n,n),float)
        g31 = np.zeros((noe,n,n,n),float)
        g32 = np.zeros((noe,n,n,n),float)
        g33 = np.zeros((noe,n,n,n),float)
        for k in range(self.Space.noe):
            g11[k,:,:,:],g12[k,:,:,:],g13[k,:,:,:],\
            g21[k,:,:,:],g22[k,:,:,:],g23[k,:,:,:],\
            g31[k,:,:,:],g32[k,:,:,:],g33[k,:,:,:] =\
                operators_f90.geometric_convection_3d(
                                        self.Space.x[k,:,:,:],
                                        self.Space.y[k,:,:,:],
                                        self.Space.z[k,:,:,:],
                                        self.Space.D)
        self.g11 = self.SpaceGL.interpolation(g11)
        self.g12 = self.SpaceGL.interpolation(g12)
        self.g13 = self.SpaceGL.interpolation(g13)
        self.g21 = self.SpaceGL.interpolation(g21)
        self.g22 = self.SpaceGL.interpolation(g22)
        self.g23 = self.SpaceGL.interpolation(g23)
        self.g31 = self.SpaceGL.interpolation(g31)
        self.g32 = self.SpaceGL.interpolation(g32)
        self.g33 = self.SpaceGL.interpolation(g33)
        
    def assemble_1d(self):
        print ''
        print '**** Assemble convection      ****'
        nz_1 = self.Space.noe * self.Space.n * self.Space.n
        nz_2 = self.SpaceGL.noe * self.SpaceGL.n * self.SpaceGL.n
        #t1 = time.clock()
        irow,jcolumn,value = \
            operators_f90.assemble_convection_1d(
                                    self.u_conv,
                                    self.Space.weights,
                                    self.Space.D,
                                    self.Space.theta,nz)
        #A = PysparseMatrix(size=self.Space.dof)
        # Assign values
        #for k in range(len(value)):
        #    i = int(irow[k])
        #    j = int(jcolumn[k])
        #    A[i,j] = A[i,j] + value[k]
        # Mask
        #A = self.Space.mask_matrix(A)
        #t2 = time.clock()
        #print '**** ex time=',t2-t1,'nnz=',A.nnz
        return A
    
    def __action_1d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_1d(self.SpaceGL.D_tilde, 
                                           self.SpaceGL.weights,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
    
                  
    def __action_x_2d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_x_2d(self.SpaceGL.I_tilde,
                                             self.SpaceGL.D_tilde, 
                                             self.SpaceGL.weights,
                                             self.g11,self.g12,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
    
    def __action_y_2d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_y_2d(self.SpaceGL.I_tilde,
                                             self.SpaceGL.D_tilde, 
                                             self.SpaceGL.weights,
                                             self.g21,self.g22,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
    
    def __action_x_3d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_x_3d(self.SpaceGL.I_tilde,
                                             self.SpaceGL.D_tilde, 
                                             self.SpaceGL.weights,
                                             self.g11,self.g12,
                                             self.g13,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
    
    def __action_y_3d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_x_3d(self.SpaceGL.I_tilde,
                                             self.SpaceGL.D_tilde, 
                                             self.SpaceGL.weights,
                                             self.g21,self.g22,
                                             self.g23,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
    
    def __action_z_3d(self,v_in):
        # Mapping
        u = self.SpaceGL.mapping_q(v_in)
        # Local matrix vector product w_L=G_L*v_L
        w = fluid_operator_f90.gradient_x_3d(self.SpaceGL.I_tilde,
                                             self.SpaceGL.D_tilde, 
                                             self.SpaceGL.weights,
                                             self.g31,self.g32,
                                             self.g33,u)
        # Mapping
        v_out = self.Space.mapping_qt(w)
        # Mask
        v_out = self.Space.mask(v_out)
        return v_out
        
    def plot(self):
        '''
        Plots the GL mesh.
        '''
            
        if self.SpaceGL.dim == 2:
            fig = plt.figure()
            ax = axes3d.Axes3D(fig)
            #u = np.zeros((self.SpaceGL.noe,self.SpaceGL.n,self.SpaceGL.n),float)
            #u = self.g11
            #for k in range(self.SpaceGL.noe):
            #    ax.plot_wireframe(self.SpaceGL.x[k,:,:],
            #                      self.SpaceGL.y[k,:,:],u[k,:,:])
            # test interpoalte
            x_gll=self.Space.x
            y_gll=self.Space.y
            u = self.w#np.zeros((self.noe,self.n+2,self.n+2),float)
            for k in range(self.Space.noe):
                ax.plot_wireframe(x_gll[k,:,:],y_gll[k,:,:],u[k,:,:])
            
            ax.set_xlabel(r'$x$')
            ax.set_ylabel(r'$y$')
            plt.show()

class Divergence:
    '''
    The divergence operator. 
    
    .. math::
    
       \underline{\mathbf D} =[\underline{D}_x\ 
                   \underline{D}_y\ \underline{D}_z] 
    
    As found in the continuity equation
    
    .. math::
    
       \\nabla\cdot\mathbf u=0\quad \Rightarrow\quad 
       \int_\Omega (\\nabla\cdot\mathbf u)q\,d\Omega=
       -b_N(q,\mathbf u)
       
       
    .. math:: 
       -b_N(q,\mathbf u)\quad \Rightarrow\quad 
       \underline{\mathbf D}\,\underline{\mathbf u}
    
    
    :argument Space: Discrete space.
    :type Space: :class:`sempy.Space` or :class:`sempy.precond.SpaceFEM`
    
    :argument SpaceGL: Discrete space.
    :type SpaceGL: :class:`sempy.fluid.SpaceGL`
    
    :attributes: * **matrix_x**, **matrix_y**, **matrix_z** 
                   (*Scipy LinearOperator*) - The 
                   components of the divergence operator 
                   :math:`\underline{\mathbf D} =[\underline{D}_x\,\underline{D}_y\, \underline{D}_z]`.
                 * **matrix** (*A list of Scipy LinearOperators*) - 
                   The divergence operator  
                   :math:`\underline{\mathbf D}`.
    
    ..               If the matrix is assembled the type is 
                   `Pysparse PysparseMatrix <http://pysparse.sourceforge.net/pysparsematrix.html#pysparse.sparse.pysparseMatrix.PysparseMatrix>`_
                   `SciPy LinearOperator <http://docs.scipy.org/doc/scipy-0.7.x/reference/sparse.linalg.html#examples>`_
                    
    **Example**::
    
       import sempy
       
       X = sempy.Space( filename = 'square', n = 6, dim = 2 )
       X_gl = sempy.fluid.SpaceGL( X )
       
       div = sempy.fluid.Divergence( X, X_gl )
       D_x = div.matrix_x
       D_y = div.matrix_y
       
    The divergence of a vector field::
    
       U = sempy.VectorFunction( X )
       D = div.matrix
       
       s = np.zeros( ( X_gl.dof ), float )
       for i in range( X.dim ):
           s = s + D[i] * U[i].glob()
                       
    '''
    def __init__(self,Space,SpaceGL):
        self.Space = Space
        self.SpaceGL = SpaceGL
        
        #if self.Space.dim == 1:
        #    self.matrix_x = spl.LinearOperator( 
        #                           ( self.SpaceGL.dof,self.Space.dof ),
        #                             matvec = self.__action_1d,
        #                             dtype = 'float' )    
        #    self.matrix = [ self.matrix_x ]
            
        if self.Space.dim == 2:
            g11,g12,g21,g22=\
               operators_f90.geometric_convection_2d(self.Space.x,
                                                     self.Space.y,
                                                     self.Space.D) 
            # Interpolate to GL mesh
            self.g11 = self.SpaceGL.interpolation(g11)
            self.g12 = self.SpaceGL.interpolation(g12)
            self.g21 = self.SpaceGL.interpolation(g21)
            self.g22 = self.SpaceGL.interpolation(g22)
            # Matrix
            self.matrix_x = spl.LinearOperator( 
                                   ( self.SpaceGL.dof,self.Space.dof ),
                                     matvec = self.__action_x_2d,
                                     dtype = 'float' )    
            self.matrix_y = spl.LinearOperator( 
                                   ( self.SpaceGL.dof,self.Space.dof ),
                                     matvec = self.__action_y_2d,
                                     dtype = 'float' )  
            self.matrix_z = 'none'
            
            self.matrix = [ self.matrix_x, self.matrix_y ]
            #spl.LinearOperator( 
            #                       ( self.SpaceGL.dof,self.Space.dof ),
            #                         matvec = self.__action_2d,
            #                         dtype = 'float' )   
            
        if self.Space.dim == 3:
            self.__geometric_3d()
            # Matrix
            self.matrix_x = spl.LinearOperator( 
                                   ( self.SpaceGL.dof,self.Space.dof ),
                                     matvec = self.__action_x_3d,
                                     dtype = 'float' )    
            # Matrix
            self.matrix_y = spl.LinearOperator( 
                                   ( self.SpaceGL.dof,self.Space.dof ),
                                     matvec = self.__action_y_3d,
                                     dtype = 'float' )   
            # Matrix
            self.matrix_z = spl.LinearOperator( 
                                   ( self.SpaceGL.dof,self.Space.dof ),
                                     matvec = self.__action_z_3d,
                                     dtype = 'float' )
            self.matrix = [ self.matrix_x, self.matrix_y, self.matrix_z ]   
            
         
              
    def __geometric_3d(self):
        # geometric 
        noe = self.Space.noe
        n = self.Space.n
        g11 = np.zeros((noe,n,n,n),float);g12 = np.zeros((noe,n,n,n),float);
        g13 = np.zeros((noe,n,n,n),float);g21 = np.zeros((noe,n,n,n),float);
        g22 = np.zeros((noe,n,n,n),float);g23 = np.zeros((noe,n,n,n),float);
        g31 = np.zeros((noe,n,n,n),float);g32 = np.zeros((noe,n,n,n),float);
        g33 = np.zeros((noe,n,n,n),float);
        for k in range(self.Space.noe):
            g11[k,:,:,:],g12[k,:,:,:],g13[k,:,:,:],\
            g21[k,:,:,:],g22[k,:,:,:],g23[k,:,:,:],\
            g31[k,:,:,:],g32[k,:,:,:],g33[k,:,:,:] =\
                operators_f90.geometric_convection_3d(
                                        self.Space.x[k,:,:,:],
                                        self.Space.y[k,:,:,:],
                                        self.Space.z[k,:,:,:],
                                        self.Space.D)
        self.g11 = self.SpaceGL.interpolation(g11)
        self.g12 = self.SpaceGL.interpolation(g12)
        self.g13 = self.SpaceGL.interpolation(g13)
        self.g21 = self.SpaceGL.interpolation(g21)
        self.g22 = self.SpaceGL.interpolation(g22)
        self.g23 = self.SpaceGL.interpolation(g23)
        self.g31 = self.SpaceGL.interpolation(g31)
        self.g32 = self.SpaceGL.interpolation(g32)
        self.g33 = self.SpaceGL.interpolation(g33)
    
    def __action_2d(self,u_in):
        # Mapping
        #u1 = self.Space.mapping_q(u_in[0].basis_coeff)
        #u2 = self.Space.mapping_q(u_in[1])
        # 
        w_x = fluid_operator_f90.divergence_x_2d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g11,self.g12,u_in[0].basis_coeff)
        w_y = fluid_operator_f90.divergence_x_2d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g21,self.g22,u_in[1].basis_coeff)
        w = w_x + w_y
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out
    
    #def __action_x_1d(self,u_in):
    #    # Mapping
    #    u = self.Space.mapping_q(u_in)
    #    # 
    #    w = fluid_operator_f90.divergence_1d(
    #                    self.SpaceGL.I_tilde,
    #                    self.SpaceGL.D_tilde,
    #                    self.SpaceGL.weights,
    #                    self.g11,self.g12,u)
    #    # Mapping
    #    v_out = self.SpaceGL.mapping_qt(w)
    #    return v_out
                
    def __action_x_2d(self,u_in):
        # Mapping
        u = self.Space.mapping_q(u_in)
        # 
        w = fluid_operator_f90.divergence_x_2d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g11,self.g12,u)
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out
    
    def __action_y_2d(self,v_in):
        # Mapping
        v = self.Space.mapping_q(v_in)
        w = fluid_operator_f90.divergence_x_2d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g21,self.g22,v)
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out
    
    def __action_x_3d(self,u_in):
        # Mapping
        u = self.Space.mapping_q(u_in)
        # 
        w = fluid_operator_f90.divergence_x_3d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g11,self.g12,self.g13,u)
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out   
    
    def __action_y_3d(self,u_in):
        # Mapping
        u = self.Space.mapping_q(u_in)
        # 
        w = fluid_operator_f90.divergence_x_3d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g21,self.g22,self.g23,u)
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out   
    
    def __action_z_3d(self,u_in):
        # Mapping
        u = self.Space.mapping_q(u_in)
        # 
        w = fluid_operator_f90.divergence_x_3d(
                        self.SpaceGL.I_tilde,
                        self.SpaceGL.D_tilde,
                        self.SpaceGL.weights,
                        self.g31,self.g32,self.g33,u)
        # Mapping
        v_out = self.SpaceGL.mapping_qt(w)
        return v_out   
    
    
'''
class Stokes:
    
    The Stokes operator. 
    
    The Stokes equation reads
    
    .. math::

        -\\nabla\cdot \mu\\nabla\mathbf u +\\nabla p &= f \\\\
        \\nabla\cdot\mathbf u &= 0
        
    which can be posed in discrete form as 
    
    .. math:: 
       :nowrap:
        
        \\begin{equation}
        \\begin{bmatrix}
        A & G\\\\
        D & 0
        \\end{bmatrix}
        \\begin{bmatrix}
        u\\\\
        p
        \\end{bmatrix}
        =
        \\begin{bmatrix}
        f\\\\
        0
        \\end{bmatrix}
        \\nonumber
        \\end{equation}
        
    where the Stokes operator is
    
    .. math::
       :nowrap:
    
        \\begin{equation}
        S=
        \\begin{bmatrix}
        A & G\\\\
        D & 0
        \\end{bmatrix}
        \\nonumber
        \\end{equation}
    
    
    
    def __init__(self,Space,SpaceGL):
        self.Space = Space
        self.SpaceGL = SpaceGL
'''


def uzawa_cg(A,b,u,tol=1e-10,max_iter=100):
    n=b.size
    p,s,r=np.zeros((n),np.float),np.zeros((n),np.float),np.zeros((n),np.float)

    s=A*u
    r=b-s
    R_new=np.inner(r,r)
    for i in range(max_iter): 
        R_old=R_new
        R_new=np.inner(r,r)
        if R_new<tol : 
            flag=0
            print 'CG: Conv. at iter.=',i,'dot(r)=',R_new
            return [u,flag]
        beta=R_new/R_old
        p=r+beta*p
        s=A*p
        Dd=np.inner(p,s)
        alpha=R_new/Dd
        u=u+alpha*p
        r=r-alpha*s
        print 'iter=',i,'r_new=',R_new
    if i == max_iter-1: 
        flag= i
        print 'cg() did not converge. dot(r)=',np.inner(r,r)
    return [u,flag]

class Uzawa:
    '''
    The Uzawa method. 
    
    The Stokes equation reads
    
    .. math:: 
       :nowrap:
        
        \\begin{equation}
        \\begin{bmatrix}
        A & G\\\\
        D & 0
        \\end{bmatrix}
        \\begin{bmatrix}
        u\\\\
        p
        \\end{bmatrix}
        =
        \\begin{bmatrix}
        b\\\\
        0
        \\end{bmatrix}
        \\nonumber
        \\end{equation}
    
    in discrete form. One way to solve this system is by using 
    the Uzawa method, i.e.:    
    
    1. Solve the Uzawa equation for pressure :math:`p` 
    
       .. math::
       
          DA^{-1}Gp = DA^{-1}b
    
    2. Solve the momentum equation for the velocity field :math:`u` 
    
       .. math::
       
          Au+Gp=b
    
    :param Space: Discrete space.
    :type Space: :class:`sempy.Space`
    :param u: Velocity
    :type u: :class:`sempy.Function`
    :param f: Forcing term
    :type f: :class:`sempy.Function`
    
    '''
    def __init__(self,Space,SpaceGL,u,b):
        self.Space = Space
        self.SpaceGL = SpaceGL
        self.u = u
        self.b = b
        
        # Laplacian
        X_fem = sempy.precond.SpaceFEM( self.Space)
        lap = sempy.operators.Laplacian( self.Space ,
                                         fem_assemble='yes')
        self.A = lap.matrix
        # Preconditioner
        A_fem = lap.matrix_fem
        P = sempy.precond.Preconditioner( self.Space, [A_fem], 
                                    drop_tol = 0.5 ).matrix
        # Gradient
        grad = sempy.fluid.Gradient( self.Space, self.SpaceGL )
        
        if self.Space.dim == 2:
            
            self.G_x = grad.matrix_x
            self.G_y = grad.matrix_y
            
        if self.Space.dim == 3:
            
            self.G_x = grad.matrix_x
            self.G_y = grad.matrix_y
            self.G_z = grad.matrix_z
        # Divergence
        div = sempy.fluid.Divergence( self.Space, self.SpaceGL )
        
        if self.Space.dim == 2:
            
            self.D_x = div.matrix_x
            self.D_y = div.matrix_y
            
        if self.Space.dim == 3:
            
            self.D_x = div.matrix_x
            self.D_y = div.matrix_y
            self.D_z = div.matrix_z
        # Solution vector
        #self.t_x = sempy.Function( self.Space, basis_coeff = 0.0 )
        #self.t_y = sempy.Function( self.Space, basis_coeff = 0.0 )
        #self.inner_solver = sempy.linsolvers.Direct(self.Space)
        self.inner_solver = sempy.linsolvers.Krylov( tol = 1e-15,
                                                     solver_type = 'bicgstab',
                                                     pre = P )
        self.outer_solver = sempy.linsolvers.Krylov(
                                            tol = 1e-08,
                                            solver_type = 'cg',
                                            pre = self.SpaceGL.mass_inv)  
        # Forcing term and linear form
        self.b_uz = self.__rhs()
        # Uzawa operator
        self.U = spl.LinearOperator( 
                          ( self.SpaceGL.dof,self.SpaceGL.dof ),
                            matvec = self.__action_uzawa,
                            dtype = 'float' )
        
        
    def solve(self):
        '''
        Solves the Sokes problem. 
        '''
        # Step 1: Pressure
        p_f = np.zeros((self.SpaceGL.dof),float)
        #[p, flag] = uzawa_cg( self.U, self.b, p_f )
        [p, flag] = self.outer_solver.solve( self.U, self.b_uz, p_f )
        #[p,flag] = uzawa_cg(self.U,self.b_uz,p_f)
        #print 'uzawa iter=',flag,'time=',self.outer_solver.ex_time
        # Step 2: Velocity
        if self.Space.dim == 2:
            b_x = self.G_x*p*(-1.0) + self.b[0]
            [u_x, flag] = self.inner_solver.solve( self.A, b_x, 
                                               self.u[0].glob() )
            self.u[0].basis_coeff = self.Space.mapping_q(u_x)
            #print 'flag=',flag
            b_y = self.G_y*p*(-1.0) + self.b[1]
            [u_y, flag] = self.inner_solver.solve( self.A, b_y, 
                                               self.u[1].glob() )
            self.u[1].basis_coeff = self.Space.mapping_q(u_y)
            return p,[self.u[0],self.u[1]],flag
        
        if self.Space.dim == 3:
            b_x = self.G_x*p*(-1.0) + self.b[0]
            [u_x, flag] = self.inner_solver.solve( self.A, b_x, 
                                               self.u[0].glob() )
            self.u[0].basis_coeff = self.Space.mapping_q(u_x)
            
            b_y = self.G_y*p*(-1.0) + self.b[1]
            [u_y, flag] = self.inner_solver.solve( self.A, b_y, 
                                               self.u[1].glob() )
            self.u[1].basis_coeff = self.Space.mapping_q(u_y)
            
            b_z = self.G_z*p*(-1.0) + self.b[2]
            [u_z, flag] = self.inner_solver.solve( self.A, b_z, 
                                               self.u[2].glob() )
            self.u[2].basis_coeff = self.Space.mapping_q(u_z)
            
            return p,[self.u[0],self.u[1],self.u[2]],flag
        
    def __rhs(self):
        print 'calc rhs...'
        if self.Space.dim == 2:
            # u component
            [t_x, flag] = self.inner_solver.solve( self.A, 
                                    self.b[0], self.u[0].glob())
            # v component
            [t_y, flag] = self.inner_solver.solve( self.A, 
                                    self.b[1], self.u[1].glob())
            # Divergence
            b = self.D_x*t_x + self.D_y*t_y
            
        if self.Space.dim == 3:
            # u component
            [t_x, flag] = self.inner_solver.solve( self.A, 
                                    self.b[0], self.u[0].glob())
            # v component
            [t_y, flag] = self.inner_solver.solve( self.A, 
                                    self.b[1], self.u[1].glob())
            # w component
            [t_z, flag] = self.inner_solver.solve( self.A, 
                                    self.b[2], self.u[2].glob())
            # Divergence
            b = self.D_x*t_x + self.D_y*t_y + self.D_z*t_z
        print '..rhs.'    
        return b
        
            
    def __action_uzawa(self,q_in):
        print 'action uzawa...'
        #
        if self.Space.dim == 2:
            # 1
            s_x = self.G_x*q_in
            s_y = self.G_y*q_in
            # 2
            t_x = sempy.Function( self.Space, basis_coeff = 0.0 )
            t_y = sempy.Function( self.Space, basis_coeff = 0.0 )
            [v_x, flag] = self.inner_solver.solve( self.A, s_x, t_x.glob() )
            [v_y, flag] = self.inner_solver.solve( self.A, s_y, t_y.glob() )
            # 3
            q_out = self.D_x * v_x + self.D_y * v_y
            
        if self.Space.dim == 3:
            # 1
            s_x = self.G_x*q_in
            s_y = self.G_y*q_in
            s_z = self.G_z*q_in
            # 2
            t_x = sempy.Function( self.Space, basis_coeff = 0.0 )
            t_y = sempy.Function( self.Space, basis_coeff = 0.0 )
            t_z = sempy.Function( self.Space, basis_coeff = 0.0 )
            [v_x, flag] = self.inner_solver.solve( self.A, s_x, t_x.glob() )
            [v_y, flag] = self.inner_solver.solve( self.A, s_y, t_y.glob() )
            [v_z, flag] = self.inner_solver.solve( self.A, s_z, t_z.glob() )
            # 3
            q_out = self.D_x * v_x + self.D_y * v_y + self.D_z * v_z
            
        print '.. uzawa.'        
        return q_out
         
        
if __name__=='__main__':
    #X = sempy.Space( filename = 'line', n = 10, dim = 1 )
    X = sempy.Space( filename = 'square', n = 10, dim = 2 )
    #X = sempy.Space( filename = 'cube', n = 4, dim = 3 )
    X_gl = sempy.fluid.SpaceGL( X )
    X_gl.plot_mesh()
    